
//---------------------------------------------------------------------------

#ifndef MachineH
#define MachineH
#include "XLCommIn.h"
#include "XLSocket.h"
#include "MachineStatusForm.h"
#include "XLCommSettings.h"

//---------------------------------------------------------------------------
class TMachine:public XLCommIn
{
private:
	TTimer * 		UpdateTimer;                        									/// Main loop timer to update the machine status
	void 				__fastcall UpdateTime(TObject *Sender); 							/// Function called when update timer fires
	TTimer * 		ConnectTimer;                       									/// Timer used to determine if communications have been established
	void 				__fastcall ConnectTime(TObject *Sender);							/// Function called when update timer fires
	AnsiString 	ConnectionStatus;								 											/// String description of connection status
	bool 				bCanStart;													 		 							/// This is true if the machine may be started
	bool 				bCanStop;														 	 								/// This is true if the machine may be stopped
	void 				__fastcall	UpdateStartStop();					 							/// Function to set the MachineStart/MachineStop flags
	bool 				bInitializingProgram;                   							/// Set to true (default on program start) until communications with XL established, steppers are initialized
	TCommSet * 	XLCommSettings;                   										/// Form that displays the XLComm status and messaging
	double 			t1;																	 									/// Time used for display of COM waiting
public:
	__fastcall 	TMachine(TComponent *Owner);
	__fastcall 	~TMachine();
 /// XLA Absorbance system
 	bool XLAAbsorbanceAvailable;																			/// Will be set true once the XLA absorbance software is in place
 /// Machine Socket
	TAOSMachineSocket * XLSocket;																			/// The machine has a TCP/IP socket to serve info
 /// Machine status form
	TMachineStatusFrm * MachineStatus;   															///Needs to be public so socket can update it
 /// Error log functions
	void 				__fastcall ShowErrorLog();														/// Show the error log, allow deletion of entries
 /// Machine db information
	AnsiString 	GUID;
	AnsiString 	Name;
	AnsiString 	SerialNumber;
	bool 				__fastcall ReadPropertiesDB();												/// Read the properties from the DB- return true on success
	bool 				__fastcall WritePropertiesDB();												/// Save the DB information- return true on success
	void 				__fastcall EditPropertiesDB();												/// Open the Properties db for editing
	void 				__fastcall ShowMachineFlags();												/// Show the status flags
 //XL settings
	XLData * 		GetXLDesired(){return XLDesired;}    									///The desired XL speed, temperature, etc. settings
	XLData * 		GetXLActual(){return XLActual;}      									///The current XL speed, temperature, etc. values
 ///Commport functions
	bool 				InitializeCommPort();                    							///Opens the commport, sends the calparms and requests settings, returns true if successful, and port open
	void				CloseCommPort();																			/// Closes the comport, sets the connection status to not connected
	void 				__fastcall ShowCommPort();       											/// Shows the commport settings, allows commands to be sent and received
	void				__fastcall SetCommPort(AnsiString s);									/// Set the commport, make sure it is in the db
	void				__fastcall DesiredSettingsChanged(bool b){						/// Sets the flag in XLDesired that the settings have been changed and need updating
								XLDesired->Changed = b;}
 /// Motor functions
	bool 				bRotorSpinning;                          							/// Set true if the software detects that the rotor is spinning
	bool 				bMachineStarted;                         							/// If the machine has been told to start, this logical is set
	bool 				bZeroingSpeed;                           							/// If true user has requested RPM to be reset to zero before starting experiment.
	bool 				bStopping;                               							/// Set to false unless the 'Stop machine' option is selected or machine state is 'Stopping'... this takes care of a problem with the Start/Stop buttons
	bool 				CanStart(){return bCanStart;}													/// Returns true if the machine may be started
	bool 				CanStop(){return bCanStop;}  													/// Returns true if the machine may be stopped
	bool 				CanSetValues();																				/// Returns true if the speed, temperature and time may be set
	void 				__fastcall UpdateZeroingSpeed();											/// Handle condition when machine is going to zero rpm
	void 				__fastcall UpdateMachineStatus();											/// Sends REQUESTSETTINGS & REQUESTSTATUS at proper time
 /// Rotor speed functions
	AnsiString 	RequestedRotorSpeed;                 									/// What is the rotor speed supposed to be. Eventually will be transmitted to the XLA  This buffers the edited text from the value being sent/recvd by the XLA
	void 				__fastcall SetRotorSpeed(int DesiredSpeed);         	/// Set the rotor speed to value in DesiredSpeed
	int					__fastcall GetDesiredRotorSpeed(){ 										/// Returns the desired rotor speed
								return XLDesired->getSpeedInt();}
	AnsiString	__fastcall GetDesiredRotorSpeedAsString(){						/// Returns the desired rotor speed as a string
								return (AnsiString)XLDesired->getSpeed();}
	int 				__fastcall GetActualRotorSpeed(){                     /// Return the Rpm as XLActual rpm
								return XLActual->getSpeedInt();}
	AnsiString  __fastcall GetActualRotorSpeedAsString(){							/// Returns the rpm as string
								return (AnsiString)GetActualRotorSpeed();}
	void				__fastcall UpdateRotorSpeed();												/// Update the current rotor speed, log any milestones
 /// Test rotor speed
	bool 				CheckForStableState(int TRpm);         							 	/// Tests whether the machine is at temperature, at vacuum and whether the Rpm is within 200 of the desired value
	bool 				IsRotorSpeedStable(int RPMDiff = 200);  						 	/// Returns true if the change rotor speed between the last and the most recent rpm measurements is less than RPMDiff
	bool 				IsAtSpeed(int RPMdes,int RPMDiff=200);   						 	/// Returns true if rotor speed has reached target RPM within tolerance of RPMDiff.
	int 				RpmChange();               													 	/// Returns the change in rpm from most recent to the last rpm
 /// Time functions
	AnsiString	__fastcall GetActualTimeAsString(){     							/// Returns the formatted time string for either remaining or elapsed
								return XLActual->getAnsiTime();}
	int					__fastcall GetActualTimeInSeconds(){
								return XLActual->getTimeSeconds();}
	long				__fastcall GetDesiredTimeHHHMM(){   									/// Return long int with time as HHHMM
								return XLDesired->getTimeHHHMM();}
	AnsiString	__fastcall GetDesiredTime(){													/// Returns formatted time or 'Hold'
								return XLDesired->getAnsiTime();}
	int					__fastcall GetDesiredTimeInSeconds(){									/// Be careful! The desired time may be 'Hold', in which case this is meaningless value
								return XLDesired->getTimeSeconds();}
	void				__fastcall SetDesiredTime(AnsiString s){							/// Set the time using a string- must be in HHHMM format
								XLDesired->putTime(s.c_str());}
 /// Acceleration/deceleration functions
	int  				__fastcall GetDesiredAccelerationAsInt();							/// Get the AnAcceleration
	int  				__fastcall GetDesiredDecelerationAsInt();							/// Get the AnDeceleration
	AnsiString  __fastcall GetDesiredAccelerationAsString();					/// Get the AnAcceleration
	AnsiString  __fastcall GetDesiredDecelerationAsString();					/// Get the AnDeceleration
	void 				__fastcall SetDesiredAcceleration(AnsiString i);			/// Set the desired acceleration
	void 				__fastcall SetDesiredDeceleration(AnsiString i);			/// Set the desired deceleration
	int  				__fastcall GetActualAccelerationAsInt();							/// Get the AnAcceleration
	int  				__fastcall GetActualDecelerationAsInt();							/// Get the AnDeceleration
	AnsiString  __fastcall GetActualAccelerationAsString();						/// Get the AnAcceleration
	AnsiString  __fastcall GetActualDecelerationAsString();						/// Get the AnDeceleration
 /// Start/stop functions
	void 				UpdateStartStatus();                     							/// Make sure the machine is started if vac ok and StartMachine issued and not spinning
	bool 				__fastcall StartMachine();               							/// Sequence of steps needed to issue a STARTFUGE command to the XLI. Returns true if STARTFUGE command is issued
	bool 				__fastcall StopMachine();                							/// Sequence of steps needed to issue the STOPFUGE command to the XLI. Returns true if the STOPFUGE commend is issued
 // Time
	double 			MachineStartTime;                      								/// Time when the STARTFUGE command was sent
	double 			StartSpinningTime;                     								/// Time in seconds when the machine first started spinning
 /// Temperature functions
	void				__fastcall SetTemperature(int temp);									/// Set the desired temperature and update the settings
	void   			UpdateTemperature();                   								/// Determine if the rotor at desired temperature for the waiting period
	double 			GetActualTemperatureAsDouble(){												/// Return a floating point value of the actual temperature
								return XLActual->getTempAsDouble();}
	int 				GetActualTemperatureAsInt(){
								return XLActual->getTempAsInt();}  									/// Returns the desired temperature as an integer
	AnsiString 	GetActualTemperatureAsString(){				    						/// Return a properly formatted temperature "xx.x" as a string
								return XLActual->getTempAsString();}
	int 				GetDesiredTemperatureAsInt(){
								return XLDesired->getTempAsInt();}  								/// Returns the desired temperature as an integer
	double 			GetDesiredTemperatureAsDouble(){           						/// Returns the desired temperature as a double
								return XLDesired->getTempAsDouble();}
	AnsiString 	GetDesiredTemperatureAsString(){				    					/// Return a properly formatted temperature "xx.x" as a string
								return XLDesired->getTempAsString();}
 /// XL Does not have access to the vacuum gauge... only have indirect access to what is going on
	void   			StartPumping();                        								/// Sets the machine to 0 rpm and begins timing how long it takes to get to 50 microns
	void   			UpdatePumping();                       								/// Test the status of pumping, set flags and update log according to the current pressure
	AnsiString 	VacuumStatus;                      										/// A string that describes the current status of the vacuum
 // Status functions
	AnsiString 	GetSysStatus();                       								/// Returns an AnsiString with the Machine status, rpm, temperature and vacuum for adding to the notes
	AnsiString 	GetMachineStatus();                										/// Display whether the XL is running, braking, waiting for vacuum, etc.
	AnsiString 	MachineType;                       										/// Is this software being used to operate an XL or an OptiMax
	AnsiString 	CurrentMachineState();             										/// Returns a string saying what the machine is waiting on
 /// Functions and values used to calculate W2t
	double LastUpdateTime;                              							/// When was the last update performed. Needed to calculate w^2t
	double w2t;                                         							/// Value calculated for w2t
	void				__fastcall UpdateW2t();																/// Update the value of w2t using the value of delta time from the last update
	void				__fastcall ResetW2t();																/// Reset the parameters used to calculate W2t
	AnsiString  GetW2tAsString(){                         						/// Returns w2t as x.xxxxEyy
								return XLActual->getAnsiW2t();}
	double 			GetW2tAsDouble(){                      								/// Return the value of omega-squared-time as a double
								return XLActual->getAnsiW2t().ToDouble();}
	double 			GetActualW2tAsDouble(){																/// Returns the value of w2t in the XLActual (from the machine)
								return ((AnsiString)(XLActual->getW2t(1))).ToDouble();}
 /// Simulation functions
	void 				SetupForSimulation();																	/// Sets the bTestingPort flag true, initializes the XLActual/Desired arrays
	void 				SetSimulation(bool b){bSimulation = b;}  							/// Set the simulation flag
	bool 				IsSimulation(){return bSimulation;}      							/// Set true if the machine is being simulated
	void 				__fastcall ShowSimulator();														/// Make the simulator form visible
 /// Communications functions
	void 				CheckDesiredSettings();                  							/// Check whether the DesiredRotorSpeed and DesiredTemperature match those on the XL. Set the XL to the Desired values
	bool 				__fastcall CommunicationsEstablished();  							/// Returns true if the XLActual and Desired settings have been obtained
	AnsiString 	ConnectionState(){return ConnectionStatus;}  					/// Returns the connection status string
 ///XML read/write
	void				__fastcall ShowMachineXML();													/// Displays form that allows the XMLRPC for the instrument to be viewed
	AnsiString 	GetDesiredXMLString(){
								return XLDesired->GetXMLString("Desired");}         /// Returns the desired settings
	void 				GetDesiredXMLStrings(TStrings * strngs){
								return XLDesired->GetXMLStrings("Desired",strngs);} /// Returns the XML strings for the desired settings
	AnsiString 	GetActualXMLString(){
								return XLActual->GetXMLString("Actual");}           /// Returns the actual settings
	void 				GetActualXMLStrings(TStrings * strngs){
								return XLActual->GetXMLStrings("Actual",strngs);};  /// Returns the XML strings for the actual settings
};
extern TMachine * Machine;
#endif
