//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "XLSimulatorFrm.h"
#include "Machine.h"
#include "Utility.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TXLSimFrm *Simulator;
//---------------------------------------------------------------------------
__fastcall TXLSimFrm::TXLSimFrm(TComponent* Owner)
	: TForm(Owner)
{
	LastTime =(double)Util->GetTimeSeconds();
	LastUpdateTime=(double)Util->GetTimeSeconds();
	w2t = 0;
}
//---------------------------------------------------------------------------
void __fastcall TXLSimFrm::SimulateNewRpm()
{///Simulate what happens when a new rotor speed is selected
	if(!bOKToAccelerate){
		LastTime = (double)Util->GetTimeSeconds();
		return;																																		/// Don't try unless it's OK to accelerate
	}
	double TestTime = (double)Util->GetTimeSeconds()-LastTime;
	if(TestTime <= 2) return;                           												/// It hasn't been two seconds since the last update
	Machine->LastRpm = Machine->GetActualRotorSpeed();																				/// Used in determining how the rotor speed is changing
	int RpmInc=TestTime*1800;                           												/// Accelerate/Decelerate at 1800 rpm/sec
	if(RpmInc > 4000) RpmInc=3300;                      												/// This can't be too large or the rtp gets messed up
	if(Machine->GetActualRotorSpeed() == 0 && TargetRpm != 0){
			Machine->StartSpinningTime=Util->GetTimeSeconds();                     	///Time in seconds when the machine first started spinning
	}
	if(Machine->GetActualRotorSpeed() > 60200)
		{///Accelerated to too high a rotor speed
			AnsiString s = (AnsiString)TargetRpm;
			Machine->XLActual->putSpeed(s.c_str());                  							 	///There's been a mistake... just get out of here!
			LastTime=(double)Util->GetTimeSeconds();        											 	///Update when the last increment in acceleration occurred
			return;
		}
	int TestRpm=Machine->GetActualRotorSpeed()-TargetRpm;
	if (TestRpm >= 100){																												/// Simulate braking
		int i = Machine->GetActualRotorSpeed() - RpmInc;																				/// Decrement the rpm
		AnsiString s = i;                                                         /// Make it easy to put it into the structure
		Machine->XLActual->putSpeed(s.c_str());
		if(Machine->GetActualRotorSpeed() < TargetRpm){                                        /// Handle case where the new rpm would be too low
		 AnsiString s = TargetRpm;
		 Machine->XLActual->putSpeed(s.c_str());
		}
	}
	if(TestRpm <= -100){																												/// Simulate acceleration
		int i = Machine->GetActualRotorSpeed() + RpmInc;                                     	/// Increment the rpm
		AnsiString s = i;                                                         /// Put in the structure
		Machine->XLActual->putSpeed(s.c_str());
		if(Machine->GetActualRotorSpeed() > TargetRpm){																				/// Handle case where rpm would be too large
		 AnsiString s = TargetRpm;
		 Machine->XLActual->putSpeed(s.c_str());
		}
	}
	if(TargetRpm==0 && Machine->GetActualRotorSpeed() < 1000){									/// You've reached zero rpm
		Machine->bRotorSpinning=false;
	}
	LastTime=(double)Util->GetTimeSeconds();                                    ///Update when the last increment in acceleration occurred
}
//---------------------------------------------------------------------------
void __fastcall TXLSimFrm::UpdateTime()
{/// Simulate elapsed time and w2t
	AnsiString s;
	double deltat=Util->GetTimeSeconds()-LastUpdateTime;
	w2t=w2t+deltat*Util->RpmToW2(Machine->GetActualRotorSpeed());
	s=s.FormatFloat("0.00E-00",w2t);
	s=s.SubString(1,1)+s.SubString(3,2)+s.SubString(6,2);
	Machine->XLActual->putW2t(s.c_str());
 /// Need to handle 2 cases for time- 1) Hold, so time = elapsed, 2) non-hold, so time = remaining
	AnsiString desiredtime = Machine->GetDesiredTime();													/// Either returns time or returns "Hold"
	int eltime=Util->GetTimeSeconds()-Machine->StartSpinningTime; 							///	Time in seconds. putTime wants HHMM
	if(desiredtime.UpperCase() == "HOLD"){																			/// Calculating the elapsed time
		s = Util->SecondsToHHHMM(eltime);
	}
	else{																																				/// Calculating remaining time
		int secs = Machine->GetActualTimeInSeconds();													/// Gets the time as a double
		secs = secs - eltime;																											/// Time remaining in seconds
		s = Util->SecondsToHHHMM(secs);	  																									/// String of # seconds 
	}
	Machine->XLActual->putTime(s.c_str());
	LastUpdateTime=Util->GetTimeSeconds();                     									/// When was the last update performed. Only need in testing mode to calculate w^2t
}
//---------------------------------------------------------------------------
void __fastcall TXLSimFrm::StopMachine()
{/// Set flags, etc. according to what happens when stop machine is issued
	TargetRpm = 0;                                       												/// Will begin deceleration
	bOKToAccelerate = Machine->VacuumSystem->bVacuumOK;  												/// Don't change the rotor speed unless the vacuum is OK
	LastTime=(double)Util->GetTimeSeconds();           													/// What was the last time the Rpm was updated
}
//---------------------------------------------------------------------------
void __fastcall TXLSimFrm::StartMachine()
{/// Set flags, etc. according to what happens when start machine is issued
	LastTime=(double)Util->GetTimeSeconds();
	bOKToAccelerate = Machine->VacuumSystem->bVacuumOK;
	if(!Machine->VacuumSystem->bPumpDown
		&& !Machine->VacuumSystem->bVacuumOK){
		Machine->StartPumping();            																				/// If we aren't pumping, then we should be
	}
	else{///Update the status of the vacuum, go to the desired rotor speed
		Machine->UpdatePumping();
		Machine->RequestedRotorSpeed = Machine->GetDesiredRotorSpeedAsString(); 		/// Set the desired rotor speed
		TargetRpm 								 = Machine->GetDesiredRotorSpeed(); 	           	/// Will begin deceleration
		bOKToAccelerate 					 = Machine->VacuumSystem->bVacuumOK;  						/// Don't change the rotor speed unless the vacuum is OK
		LastTime		= (double)Util->GetTimeSeconds();       												/// What was the last time the Rpm was updated
	}
}
//---------------------------------------------------------------------------
void __fastcall TXLSimFrm::StartPumping()
{/// Simulate what happens when the pumps are started
	TargetRpm=Machine->GetDesiredRotorSpeed();						 	/// Set the target speed to the desired speed
	bOKToAccelerate=Machine->VacuumSystem->bVacuumOK;      	/// Don't change the rotor speed unless the vacuum is OK
	LastTime=(double)Util->GetTimeSeconds()-3;    					/// What was the last time the Rpm was updated... force acceleration to start
}
//---------------------------------------------------------------------------
void __fastcall TXLSimFrm::CheckDesiredSettings()
{/// Simulate what happens when the desired settings are checked for xmission to XL
	TargetRpm=Machine->GetDesiredRotorSpeed();
	bOKToAccelerate=Machine->VacuumSystem->bVacuumOK;       ///Don't change the rotor speed unless the vacuum is OK
	LastTime=(double)Util->GetTimeSeconds();                ///What was the last time the Rpm was updated
}
//---------------------------------------------------------------------------
AnsiString 	__fastcall TXLSimFrm::GetMachineStatus()
{/// Returns the simulated machine status string
	if(Machine->IsAtSpeed(Machine->GetDesiredRotorSpeed(),100)
		&& Machine->VacuumSystem->bVacuumOK){
		 return "Running";                           					/// All is stable, return running state
	}
	else{																										/// Not at a stable speed, determine whether accelerating or decelerating
		if(!Machine->VacuumSystem->bVacuumOK
			|| Machine->VacuumSystem->bPumpDown){
				return "Power on";                                /// Pumping down or at air
		}
		if((Machine->GetDesiredRotorSpeed() - Machine->GetActualRotorSpeed()) > 100
			&& !Machine->bStopping){
				return "Accelerating";                           	/// Accelerating... waiting to get to speed
		}
		if((Machine->GetDesiredRotorSpeed() - Machine->GetActualRotorSpeed()) < 100
			|| (Machine->bStopping && Machine->GetActualRotorSpeed() != 0)){
				return "Decelerating";                           	/// Decelerating or stopping
		}
		if(Machine->GetActualRotorSpeed() == 0 && TargetRpm == 0){
				return "Power On";                               	/// Rotor at rest
    }
		return "Power On";
	}
}
//---------------------------------------------------------------------------
void __fastcall TXLSimFrm::FormShow(TObject *Sender)
{///Update the values
	UpdateDisplay();
}
//---------------------------------------------------------------------------
void __fastcall TXLSimFrm::UpdateDisplay()
{/// Update form contents
	if(!Visible) return;																												///No need to update the form if it isn't showing
	TargetRpmTxt->Caption 		= (AnsiString)TargetRpm;
	LastRpmUpdateTxt->Caption = (AnsiString)(double)Util->GetTimeSeconds()-LastTime;
	Lastw2tUpdateTxt->Caption = (AnsiString)(double)Util->GetTimeSeconds()-LastUpdateTime;
	w2tTxt->Caption 					= (AnsiString)FormatFloat("0.00E-00",w2t);
	OkAcCB->Checked						= bOKToAccelerate;
}
//---------------------------------------------------------------------------
void __fastcall TXLSimFrm::BitBtn2Click(TObject *Sender)
{
	Hide();
}
//---------------------------------------------------------------------------
void __fastcall TXLSimFrm::BitBtn1Click(TObject *Sender)
{
	UpdateDisplay();
}
//---------------------------------------------------------------------------
