//---------------------------------------------------------------------------
#include <math.h>
#include "Utility.h"
#include "VerInfo.h"
#pragma package(smart_init)

Utility *Util;
//---------------------------------------------------------------------------
__fastcall Utility::Utility(TComponent *Owner):TComponent(Owner)
{
   //Added 7/02
    MainStatusBar = new TXLStatusBar(this);
  //Added 10/02
    FlashBox=new TFlashBox(this);
    FlashBox->Hide();                 ///Make sure this is inactive
}
//---------------------------------------------------------------------------
__fastcall Utility::~Utility()
{
	delete MainStatusBar;
  delete FlashBox;
}
 //===========================================================================
 ///Simple math functions
 //---------------------------------------------------------------------------
double __fastcall Utility::RoundTo(double value, int sigdigits)
{/*Round to a specific digit or power of ten
 sigdigits has a valid range of 37 to -37.  Here are some valid examples
  of sigdigits values...
   3 = 10^3  = 1000   = thousand's place
   2 = 10^2  =  100   = hundred's place
   1 = 10^1  =   10   = ten's place
  -1 = 10^-1 = 1/10   = tenth's place
  -2 = 10^-2 = 1/100  = hundredth's place
  -3 = 10^-3 = 1/1000 = thousandth's place        */
  if(sigdigits > 37 || sigdigits < -37) return value;
  double LFactor;
  LFactor = pow10(sigdigits);
  return (double)DoubleToRoundedInt(value/LFactor)*LFactor;
}
//---------------------------------------------------------------------------
int __fastcall Utility::LetterToNumber(AnsiString s)
{///Returns the number corresponding to the letter, A=0, B=1, etc.
  s=s[s.Length()];    //Get the channel letter... A, B, C, etc.
  char *s1=s.c_str();
  return (s1[0]-'A');
}
//---------------------------------------------------------------------------
AnsiString  __fastcall Utility::NumberToLetter(int Number)
{///Returns the letter corresponding to a number A=0, B=1, etc.
  char s='A'+Number;              //Letter = A+Number
  return (AnsiString)s;
}
//---------------------------------------------------------------------------
int __fastcall Utility::DoubleToRoundedInt(double d)
{///Convert a double to an integer, rounding up if decimal > 5
  int i;
  i=(int)d;
  d=10*d;         ///Rotate the first decimal place to the last digit
  int j;
  j=fmod(d,10);   ///Get the remainder when d is re-rotated back to before the decimal
  if(j>=5) ++i;   ///If the remainder is equal to or greater than 5, then round up i
  return i;
}
//---------------------------------------------------------------------------
void __fastcall Utility::AverageScans(int numvalues,long double * values, long double * average,long double * stdev)
{
//  int iTempI;
  int j=0;
  long double * av=&values[0];
  long double avv=0.0;
  long double std=0.0;
  long double nscansq= (long double)numvalues*(long double)numvalues - numvalues;
  long double nscnm1=(long double)numvalues - 1.0;
  if (nscansq==0) nscansq=1.0;
  if (nscnm1==0) nscnm1=1.0;
  for(j=0;j<numvalues;j++) //Grab as many scans as needed for the average
   {
    avv=avv + *(av+j);       //accumulate intensities
    std=std + (*(av+j))*(*(av+j));  //accumulate square of intensities
   }
  j=5;//dummy
  std = (std/nscnm1)-((avv*avv)/nscansq);
 *stdev = sqrtl(std/numvalues);
 *average = avv/numvalues;
}
//----------------------------------------------------------------------------
double __fastcall Utility::SumEm(int numvalues,double * values)
{
 int i=0;
 double* pV=&values[0];
 long double pS=0.0;
 int * pN=&numvalues;
 for(i=0;i< *pN;i++)
  {
	  pS=pS+ (long double)*(pV+i);
  }
 return((double)pS);
}
//----------------------------------------------------------------------------
double __fastcall Utility::SumEm(int numvalues,unsigned int * values)
{
 int i=0;
 unsigned int* pV=&values[0];
 long double pS=0.0;
 int * pN=&numvalues;
 for(i=0;i< *pN;i++)
  {
	  pS=pS+ (long double)*(pV+i);
  }
 return((double)pS);
}
//---------------------------------------------------------------------------
double __fastcall Utility::Mean(double *values, int StartIndex, int EndIndex)
{ // Compute average of values and return as double.
  int numvalues = EndIndex - StartIndex +1;
  if(StartIndex < 0 || EndIndex < StartIndex) return 0.;
  int i;
  double sum, average;
  sum=0.;
  for(i=StartIndex; i<=EndIndex; i++){
    sum += values[i];
  }
  average = sum/(double)numvalues;
  return  average;
}
//---------------------------------------------------------------------------
int __fastcall Utility::Mean(int *values, int StartIndex, int EndIndex)
{ // Compute average of values and return as double.
  int numvalues = EndIndex - StartIndex +1;
  if(StartIndex < 0 || EndIndex < StartIndex) return 0;
  int i;
  double sum, average;
  sum=0.;
  for(i=StartIndex; i<=EndIndex; i++){
    sum += values[i];
  }
  average = sum/(double)numvalues;
  return  (int)average;
}
//---------------------------------------------------------------------------
unsigned short __fastcall Utility::Mean(unsigned short *values, int StartIndex, int EndIndex)
{ // Compute average of values and return as double.
  int numvalues = EndIndex - StartIndex +1;
  if(StartIndex < 0 || EndIndex < StartIndex) return 0;
  int i;
  double sum, average;
  sum=0.;
  for(i=StartIndex; i<=EndIndex; i++){
    sum += values[i];
  }
  average = sum/(double)numvalues;
  return  (unsigned short)average;
}
//---------------------------------------------------------------------------
double __fastcall Utility::Mean(double *values, int numvalues)
{ // Compute average of values and return as double.
  if(numvalues <= 0) return 0.;
  double sum, average;
  int i;
  sum=0.;
  for(i=0; i<numvalues; i++){
    sum += values[i];
  }
  average = sum/(double)numvalues;
  return  average;
}
//---------------------------------------------------------------------------
int __fastcall Utility::Mean(int *values, int numvalues)
{ // Compute average of values and return as int.
  if(numvalues <= 0) return 0.;
  int i;
  double sum,average;
  sum=0.;
  for(i=0; i<numvalues; i++){
    sum += values[i];
  }
  average = sum/(double)numvalues;
  return  (int)average;
}
//---------------------------------------------------------------------------
unsigned short __fastcall Utility::Mean(unsigned short *values, int numvalues)
{ // Compute average of values and return as unsigned short.
  if(numvalues <= 0) return 0.;
  int i;
  double sum, average;
  sum=0.;
  for(i=0; i<numvalues; i++){
    sum += values[i];
  }
  average = sum/(double)numvalues;
  return  (unsigned short)average;
}
//---------------------------------------------------------------------------
bool __fastcall Utility::MeanAndStdDeviation(double *values1,int numvalues,double *average,double *stdev)
{///Calculate meand and standard deviation for an array of doubles
  bool ret;
  ret=false;
  if(numvalues <= 0) return ret;  ///Flag error- no values to average
  int j;
  double *av=&values1[0];
  double avv=0.0;
  double std=0.0;
  try{
    for(j=0;j<numvalues;j++){            //Grab as many scans as needed for the average
      avv=avv + *(av+j);                 //accumulate intensities
    }
    // Compute average
    *average = avv/(double)numvalues;
    double diff;
    for(j=0;j<numvalues;j++){
    //Compute squared difference from mean.
      diff =  (*(av+j) - *average);
      std += diff*diff;
    }
    *stdev=sqrt(std/numvalues);
    ret=true;
  }
  catch(...)
    {
     *stdev = 0;
     *average = 0;
     ret=false;
    }
  return ret;
}
//---------------------------------------------------------------------------
bool __fastcall Utility::MeanAndStdDeviation(double *values1,int start,int end,double *average,double *stdev)
{///Calculate the mean and standard deviation on part of an array of doubles
  bool ret;
  ret=false;
  int numvalues = end-start + 1;
  if(numvalues <= 2) return ret;  ///Flag error- no values to average
  int j;
  double *av=&values1[start];
  double avv=0.0;
  double std=0.0;
  try{
    for(j=0;j<numvalues;j++){            //Grab as many scans as needed for the average
      avv=avv + (double)*(av+j);                      //accumulate intensities
    }
    // Compute average
    *average = avv/(double)numvalues;
    double diff;
    for(j=0;j<numvalues;j++){
    //Compute squared difference from mean.
      diff =  (*(av+j) - *average);
      std += diff*diff;
    }
    *stdev=sqrt(std/numvalues);
    ret=true;
  }
  catch(...){
   *stdev = 0;
   *average = 0;
   ret=false;
  }
  return ret;
}
//---------------------------------------------------------------------------
bool __fastcall Utility::MeanAndStdDeviation(unsigned short *values1,int start,int end,double *average,double *stdev)
{///Calculate the mean and standard deviation on part of an array of unsigned shorts
  bool ret;
  ret=false;
  int numvalues = end-start + 1;
  if(numvalues <= 2) return ret;  ///Flag error- no values to average
  int j;
  unsigned short *av=&values1[start];
  double avv=0.0;
  double std=0.0;
  try{
    for(j=0;j<numvalues;j++){            //Grab as many scans as needed for the average
      avv=avv + (double)*(av+j);                      //accumulate intensities
    }
    // Compute average
    *average = avv/(double)numvalues;
    double diff;
    for(j=0;j<numvalues;j++){
    //Compute squared difference from mean.
      diff =  (*(av+j) - *average);
      std += diff*diff;
    }
    *stdev=sqrt(std/numvalues);
    ret=true;
  }
  catch(...){
   *stdev = 0;
   *average = 0;
   ret=false;
  }
  return ret;
}
//---------------------------------------------------------------------------
bool __fastcall Utility::MeanAndStdDeviation(int *values1,int numvalues,double *average,double *stdev)
{///Calculate meand and standard deviation for an array of integers
  bool ret;
  ret=false;
  if(numvalues <= 0) return ret;  ///Flag error- no values to average
  int j;
  int *av=&values1[0];
  double avv=0.0;
  double std=0.0;
  try{
    for(j=0;j<numvalues;j++){            //Grab as many scans as needed for the average
      avv=avv + *(av+j);                 //accumulate intensities
    }
    // Compute average
    *average = avv/(double)numvalues;
    double diff;
    for(j=0;j<numvalues;j++){
    //Compute squared difference from mean.
      diff =  (*(av+j) - *average);
      std += diff*diff;
    }
    *stdev=sqrt(std/numvalues);
    ret=true;
  }
  catch(...)
    {
     *stdev = 0;
     *average = 0;
     ret=false;
    }
  return ret;
}
//----------------------------------------------------------------------------
bool __fastcall Utility::Delay(unsigned wait) //wait is the number of milliseconds to pause
{
  if (wait==0) return 0;  //Minimize delay
  double start, stop;
  start=GetTimeMilliSeconds();
	do
	{
      stop=GetTimeMilliSeconds();
	} while (stop-start<wait);
  return 1;
}
//---------------------------------------------------------------------------
double __fastcall Utility::GetTimeMilliSeconds()    //Return time in msec
{
  double dt;
  unsigned short Hour, Min, Sec, MSec;
  TDateTime dtPresent = Now();                  /// Get current time
  dt=(int)dtPresent;
  DecodeTime(dtPresent, Hour, Min, Sec, MSec);  /// Convert time to unsigned shorts
  dt=24*dt;                                     /// Convert days to hours
  dt=3600*dt;                                   /// Convert days to seconds
  dt=dt+3600*Hour;                              /// Convert hours to seconds
  dt = dt + 60*Min;                             /// Convert minutes to seconds
  dt = dt + Sec;
  dt=1000*dt+MSec;
  return dt;
}
//---------------------------------------------------------------------------
void __fastcall Utility::FirstTime()
{
  StartTime=GetTimeMilliSeconds();
  EndTime=StartTime;
  LastTime=StartTime;
  NumberOfPases=1;
}
//---------------------------------------------------------------------------
void __fastcall Utility::ResetTime()
{
  StartTime=0;
  LastTime=0;
  EndTime=0;
  AccumulatedTime=0;
  CalibrationTime=0;
  NumberOfPases=0;
}
//---------------------------------------------------------------------------
void __fastcall Utility::NextTime()
{
  EndTime=GetTimeMilliSeconds();
  LastTime=StartTime;
  AccumulatedTime=LastTime-StartTime-CalibrationTime;
  NumberOfPases++;
}
//===========================================================================
///Smoothing, derivative, min/max and peak finding functions
//---------------------------------------------------------------------------
bool __fastcall Utility::BoxCar(double * X,int boxsize,int num)
{///Perform a boxcar smoothing on the array
	if(fmod(boxsize,2) == 0) boxsize = boxsize+1;                               ///Make sure the boxsize is odd number
	if(num <= boxsize
		|| num == 0
		|| boxsize < 3) return false;                                             ///Nothing to do
	int first = 0;
	int last  = boxsize;
	int midpt = boxsize/2;                                                      ///The index that will be replaced with the average
	double sum=0;                                                               ///Running sum
	for(int i=0;i<boxsize;++i)                                                  ///Get the sum for the first datapoints
		sum = sum +X[i];                                                          ///Initializes sum
	for(int i = 0; i < num-boxsize; i++)
		{///Use boxcar to roll-off signal
			X[midpt++] = sum/boxsize;
			sum = sum - X[first++] + X[last++];
		}
	return true;
}
//---------------------------------------------------------------------------
//  Reverse Smoothing Algorithm
void  __fastcall Utility::RevSmooth(double* x, double* y, int n)
{
	double* S = new double [n+1];
	double b = 0;
  int i;
  for (i=1; i<=n; i++)	{	// store the original array and find the average
		S[i] = y[i];
		b += y[i];
  }
	b /= n;
	for (i=2; i<n; i++)  {		// estimate the derivatives
		y[i] = 3*(x[i-1]*S[i-1] + x[i]*S[i] + x[i+1]*S[i+1])
			 - (x[i-1] + x[i] + x[i+1]) * (S[i-1] + S[i] + S[i+1]);
		double D;
    D = 3*(x[i-1]*x[i-1] + x[i]*x[i] + x[i+1]*x[i+1])
		  - (x[i-1] + x[i] + x[i+1]) * (x[i-1] + x[i] + x[i+1]);
		y[i] /= D;
	}				// estimate end point derivatives
  y[1] = y[2] + (y[3]-y[5])*(x[2]-x[1]) / (x[5]-x[3]);
	y[n] = y[n-1] + (y[n-2]-y[n-4])*(x[n]-x[n-1]) / (x[n-2]-x[n-4]);

	for (i=2; i<=n; i++)		// forward pass, exponential smoothing
		y[i] = (y[i] + y[i-1]) / 2;
	for (i=n-1; i>=1; i--)		// reverse pass exponential smoothing
		y[i] = (y[i] + 2*y[i+1]) / 3;
	for (i=2; i<=n; i++)		// forward pass
		y[i] = (y[i] + y[i-1]) / 2;

	double c;
  c = S[1] = y[1];	// integrate the derivatives
	for (i=2; i<=n; i++)	{
		c += (2*y[i]+y[i-1]) * (x[i]-x[i-1]) / 3;
		S[i] = c;
	}
	double d;
  d = 0;			// find integration constant
	for (i=1; i<=n; i++)  {
		y[i] = S[i];
		d += y[i];
	}
	d /= n;
	for (i=1; i<=n; i++)
		y[i] += (b - d);
  delete [ ] S;			// cleanup
}
//---------------------------------------------------------------------------
bool  __fastcall Utility::FindMinMax(unsigned short *Data, int StartIndx, int EndIndx, int *Mindex, int *Maxdex)
{///Find the index of the maximum and minimum in the Data array
  if(StartIndx >= EndIndx)
    {///Make sure you can do this
      *Mindex=StartIndx;
      *Maxdex=StartIndx;
      return false;
    }
  int i;
  int NPts = (EndIndx - StartIndx) + 1;
  double * Smoothx;
  Smoothx = new double[NPts];
  double * Smoothy;
  Smoothy = new double[NPts];
  for(int i=0; i<NPts; i++)
    {
      Smoothx[i]=i;
      Smoothy[i]=(double)Data[StartIndx+i];
    }
  RevSmooth(Smoothx,Smoothy,NPts-1);
  double PosPeak;
  double NegPeak;
  PosPeak=Data[StartIndx];
  NegPeak=Data[EndIndx];
  *Mindex=StartIndx;
  *Maxdex=StartIndx;
  for(i=0; i<NPts; i++)
    {///Go over the array and find the min and max
      if(Smoothy[i] == Smoothy[i])
        {// make sure the value is a real number
          if(Smoothy[i]>PosPeak)
            {
              PosPeak = Smoothy[i];
              *Maxdex=StartIndx+i;
            }
          if(Smoothy[i]<NegPeak)
            {
              NegPeak = Smoothy[i];
              *Mindex=StartIndx+i;
            }
        }
    }
  if(Smoothx) delete[] Smoothx;
  if(Smoothy) delete[] Smoothy;
  return true;
}
//---------------------------------------------------------------------------
bool  __fastcall Utility::FindMinMax(unsigned short *Data, int StartIndx, int EndIndx, int *Mindex, int *Maxdex, bool bSmooth)
{///Find the index of the minimum and maximum in the Data array. Use the smoothing form if asked to.
  if(bSmooth)
    {//User wants to use the smoothed data form
      if(FindMinMax(Data, StartIndx, EndIndx, Mindex, Maxdex)) return true;
      else return false;
    }
  else
    {//User will work with unsmoothed data
      if(StartIndx >= EndIndx)
        {//Check to make sure the user hasn't chosen bogus portion of the array
          *Mindex=StartIndx;
          *Maxdex=StartIndx;
          return false;
        }
      int i;
      unsigned short PosPeak;
      unsigned short NegPeak;
      PosPeak=Data[StartIndx];
      NegPeak=Data[EndIndx];
      *Mindex=StartIndx;
      *Maxdex=StartIndx;
      for(i=StartIndx; i<EndIndx; i++)
        {
          if(Data[i] == Data[i])
            {// make sure the value is a real number
              if(Data[i]>PosPeak)
                {
                  PosPeak = Data[i];
                  *Maxdex=i;
                }
              if(Data[i]<NegPeak)
                {
                  NegPeak = Data[i];
                  *Mindex=i;
                }
            }
        }
    }
  return true;
}
//---------------------------------------------------------------------------
bool  __fastcall Utility::PositiveEdgeFinder(unsigned short *Data, int StartIndx, int EndIndx, int *index,int PointsInTransform)
{///Find the index of the leading edge of a pulse
  int Count = EndIndx - StartIndx;
  *index=StartIndx;
  if(Count <= 0
    || StartIndx+PointsInTransform > EndIndx)
    {///Make sure this can be done
      return false;
    }
  int midpt=PointsInTransform/2;                                              ///Midpoint of the transform, corresponding to the data point being transformed
  int i;                                                                      ///Loop variable over data set
  int Psum=0;
  int Nsum=0;
  for(i=StartIndx;i<StartIndx+PointsInTransform;i++)
    {///Starting- must deal with transforming the first few data points separately
      Nsum=Nsum+Data[i];                                                       ///Transform data points between 0 and the length of the transform array separately
      Psum=Psum+Data[midpt+i];                                                 ///Transform data points between 0 and the length of the transform array separately
    }
  int Xform = Psum-Nsum;                                              ///Calc the xform
 ///Set the indices needed to point to the various elements being manipulated in the arrays
  int Pstart=StartIndx;                                                               ///First point in the data array is the first point in the 'positive' array
  int Nstart=StartIndx+midpt;                                                           ///Midpoint of the transform is the first element in the sum to be subtracted
  int Nend=StartIndx+PointsInTransform;                                                 ///Last point in the array that gets subtracted
  for(i=StartIndx+midpt;i<EndIndx-midpt;i++)                                              ///Loop over the main part (middle) of the data set
   {///Do mid-data array
     Nsum=Nsum-Data[Pstart]+Data[Nstart];                                       ///Get the Psum
     Psum=Psum-Data[Nstart]+Data[Nend];                                         ///Get the Nsum
     int T_Rtp = Psum-Nsum;                                           ///Calc the xform
     if(T_Rtp > Xform)
      {///Found a better point
        *index = Nstart;                                                      ///Set the index to be at the mid point of xform
        Xform=T_Rtp;                                                          ///Set the new test value
      }
    ++Pstart;
    ++Nstart;
    ++Nend;
   }
  return true;
}
//---------------------------------------------------------------------------
bool  __fastcall Utility::NegativeEdgeFinder(unsigned short *Data, int StartIndx, int EndIndx, int *index,int PointsInTransform)
{///Find the index of the leading edge of a pulse
  int Count = EndIndx - StartIndx;
  *index=StartIndx;
  if(Count <= 0
    || StartIndx+PointsInTransform > EndIndx)
    {///Make sure this can be done
      return false;
    }
  int midpt=PointsInTransform/2;                                              ///Midpoint of the transform, corresponding to the data point being transformed
  int i;                                                                      ///Loop variable over data set
  int Psum=0;
  int Nsum=0;
  for(i=StartIndx;i<StartIndx+PointsInTransform;i++)
    {///Starting- must deal with transforming the first few data points separately
      Psum=Psum+Data[i];                                                       ///Transform data points between 0 and the length of the transform array separately
      Nsum=Nsum+Data[midpt+i];                                                 ///Transform data points between 0 and the length of the transform array separately
    }
  int Xform = Psum-Nsum;                                                      ///Calc the xform
 ///Set the indices needed to point to the various elements being manipulated in the arrays
  int Pstart=StartIndx;                                                       ///First point in the data array is the first point in the 'positive' array
  int Nstart=StartIndx+midpt;                                                 ///Midpoint of the transform is the first element in the sum to be subtracted
  int Nend=StartIndx+PointsInTransform;                                       ///Last point in the array that gets subtracted
  for(i=StartIndx+midpt;i<EndIndx-midpt;i++)                                  ///Loop over the main part (middle) of the data set
   {///Do mid-data array
     Psum=Psum-Data[Pstart]+Data[Nstart];                                     ///Get the Psum
     Nsum=Nsum-Data[Nstart]+Data[Nend];                                       ///Get the Nsum
     int T_Rtp =Psum-Nsum;                                                    ///Calc the xform
     if(T_Rtp > Xform)
      {///Found a better point
        *index = Nstart;                                                      ///Set the index to be at the mid point of xform
        Xform=T_Rtp;                                                          ///Set the new test value
      }
    ++Pstart;
    ++Nstart;
    ++Nend;
   }
  return true;
}
//---------------------------------------------------------------------------
bool   __fastcall Utility::FindPulse(double * Data,int NPts, int PulseWidth,double *bestsum,int *bestindex)
{///Use simple sum of PulseWidth points slid across IntensityData, sets bestsum and best index to midpoint of pulse. Return true if successful, else returns false
  if(NPts==0                                                ///Bail out if this cannot be done
    || PulseWidth==0
    || NPts<PulseWidth) return false;
 ///Initialize the bestsum with the fir
  PulseWidth=PulseWidth/2;
  double sum  = SumEm(PulseWidth,&Data[0]);
  *bestsum=sum;
  *bestindex= PulseWidth;
  for(int i=PulseWidth;i< NPts-PulseWidth;++i)
    {///Simply find the sum of the intensities for a swath equal to the size of sector width
      sum=sum-Data[i]+Data[i+PulseWidth];
      if(sum > *bestsum)
        {///Keep the largest sum and starting index of the intensity range
          *bestsum=sum;
          *bestindex=i;
        }
    }
  return true;
}
//---------------------------------------------------------------------------
bool  __fastcall Utility::FindPeak(double *Data, int NPts, int direction, int *PeakLocation, int *FWHM)
{/// Finds location of largest peak (positive/negative).  If direction is
  /// +1 the peak found will be a positive going peak, if direction is -1
  /// the peak will be a negative going peak.  The PeakLocation is the bin number
  /// in which the maximum was found.  The FWHM is the width of the peak in
  /// integer bins.
  double temp=0;
  int FWHM1=0;
  int FWHM2=0;
  *FWHM = -1;
  *PeakLocation=-1;
  if(NPts <=0 ) return false;
  int LocalPeakLocation;
  int j;
  double * Smoothx;
  Smoothx = new double[NPts];
  double * Smoothy;
  Smoothy = new double[NPts];
  for(int i=0; i<NPts; i++)
    {
      Smoothx[i]=i;
      Smoothy[i]=Data[i];
    }
  RevSmooth(Smoothx,Smoothy,NPts-1);
  if(direction > 0)
    {/// Positive going peak
      for(int i=0; i<NPts; i++)
        {
          if(Smoothy[i] > temp)
            {
              temp=Smoothy[i];
              LocalPeakLocation=i;
            }
        }
      *PeakLocation=LocalPeakLocation;
      /// Now look for FWHM
      j= LocalPeakLocation;
      /// Start at peak and go backward until find FWHM1
      while(Smoothy[j] > Smoothy[LocalPeakLocation]/2.)
        {
          FWHM1=j;
          if(j <= 0) break;
          --j;
        }
      /// Start at peak and go forward until find FWHM2
      j=LocalPeakLocation;
      while(Smoothy[j] > Smoothy[LocalPeakLocation]/2.)
        {
          FWHM2=j;
          if(j >= NPts) break;
          ++j;
        }
      if(FWHM1 != -1 && FWHM2 != -1) *FWHM = FWHM2-FWHM1;
      else *FWHM=-1;
    }
  else
    {/// Negative going peak
      for(int i=0; i<NPts; i++)
        {
          if(Smoothy[i] < temp)
            {
              temp=Smoothy[i];
              LocalPeakLocation=i;
            }
        }
      *PeakLocation=LocalPeakLocation;
      /// Now look for FWHM
      j=LocalPeakLocation;
      /// Start at peak and go backward until find FWHM1
      while(Smoothy[j]<Smoothy[LocalPeakLocation]/2.)
        {
          FWHM1=j;
          if(j <= 0) break;
          --j;
        }
      /// Start at peak and go forward until find FWHM2
      j=LocalPeakLocation;
      while(Smoothy[j]<Smoothy[LocalPeakLocation]/2.)
        {
          FWHM2=j;
          if(j >= NPts) break;
          ++j;
        }
    }
  *FWHM=(FWHM2-FWHM1) - 1; /// Subtract 1/2 cell on both sides to get more accurate FWHM
  delete [] Smoothy;
  delete [] Smoothx;
  return true;
}
//---------------------------------------------------------------------------
bool  __fastcall Utility::CalcDeriv(double *Data, int NPts, double *Derivative)
{/// Computes the derivative of the Data.
  /// Before calculating the derivative the Data is smoothed using the Reverse Smoothing
  /// algorithm in Utility.
  if(NPts <= 0) return false;
  double * Smoothx;
  Smoothx = new double[NPts];
  double * Smoothy;
  Smoothy = new double[NPts];
  for(int i=0; i<NPts; i++)
    {
      Smoothx[i]=i;
      Smoothy[i]=Data[i];
    }
  RevSmooth(Smoothx,Smoothy,NPts-1);
  for(int k=0;k<NPts; k++)
    {
      if(k>=3 && k<NPts-4)
        {
          Derivative[k]=((Smoothy[k+3]+Smoothy[k+2]+Smoothy[k+1]) -
                        (Smoothy[k-3]+Smoothy[k-2]+Smoothy[k-1]))/6.;
        }
      else
        {
          Derivative[k]=0;
        }
    }
  delete [] Smoothy;
  delete [] Smoothx;
  return true;
}
//---------------------------------------------------------------------------
bool  __fastcall Utility::CalcDeriv(double *Data, int NPts, double *Derivative, bool bSmooth)
{/// Overloaded version of CalcDeriv.  Allows user to specify whether to smooth the
  /// input data or not.  If bSmooth = true then it calls CalcDeriv with smoothing.
  /// otherwise it computes the derivative of the raw data.
  if(NPts <= 0) return false;
  if(bSmooth)
    {///Use smoothed data to determine derivative
      if(CalcDeriv(Data, NPts, Derivative)) return true;
      else return false;
    }
  else
    {///Use unsmoothed data to determine derivative
      for(int k=0;k<NPts; k++)
        {
          if(k>=3 && k<NPts-4)
            {
              Derivative[k]=((Data[k+3]+Data[k+2]+Data[k+1]) -
                            (Data[k-3]+Data[k-2]+Data[k-1]))/6.;
            }
          else
            {
              Derivative[k]=0;
            }
        }
    }
  return true;
}
//===========================================================================
///Validate floating point
//---------------------------------------------------------------------------
bool __fastcall Utility::CheckValidFloat(AnsiString s)
{///Returns true if s is a valid floating point number
	int hasdecimal=0;
	for(int i=1; i<=s.Length(); i++)
		{
			char ss=(char)s[i];
			if(ss < 48 || ss > 57)
				{
					if(ss == 46)
						{
							++hasdecimal;
							if(hasdecimal > 1) return false;
						}
					else return false;
				}
		}
	return true;
}
//===========================================================================
///File functions
//---------------------------------------------------------------------------
bool __fastcall Utility::IsFloFile(AnsiString filename)
{///Returns true if the file has FI extension
	AnsiString ext=UpperCase(ExtractFileExt(filename));
	if(ext.SubString(2,2)== "FI" ) return true;
	return false;
}
//---------------------------------------------------------------------------
AnsiString __fastcall Utility::GetUniqueFileName(AnsiString filename)
{///Check that there is no existing file with this name. If there is, create a unique name
	if(!FileExists(filename)) return filename;  ///This file name does not already exist, so just return it for re-use
	int i;
	i=1;
	AnsiString s;
	s=filename;
	AnsiString fn;
	fn=ExtractFileName(filename);
	AnsiString fp;
	fp=ExtractFilePath(filename);
	while(FileExists(s))
		{
			s=fp+(AnsiString)i+"_"+fn;
			i++;
		}
 return s;
}
///==========================================================================
// Time formatting utilities
//---------------------------------------------------------------------------
double __fastcall Utility::RpmToW(int rpm)
{/// Convert rpm to omega
	double d;
	d = rpm*M_PI/30;
	return d;
}
//---------------------------------------------------------------------------
double __fastcall Utility::RpmToW2(int rpm)
{/// Convert rpm to omega square
	double d;
	d = rpm*M_PI/30;
	return d*d;
}
//---------------------------------------------------------------------------
AnsiString __fastcall Utility::SetScanNumOrTime(int value)
{///Set the text of the box to either number of scans (if value > 0) or time to scan (value < 0)
	AnsiString s;
	if(value > 0) s=(AnsiString)value;
	else s=IntToHM(-value);
	return s;
}
//---------------------------------------------------------------------------
int __fastcall Utility::TimeToInt(AnsiString s)
{///Convert either 00:00 or 00:00:00 to seconds
	int pos = s.Pos(":");
	AnsiString test = s.SubString(pos+1,s.Length());
	if(test.Pos(":") == 0) return HMToInt(s);
	else return HMSToInt(s);
}
//---------------------------------------------------------------------------
int __fastcall Utility::HMToInt(AnsiString s)
{///Convert from 00:00 or 000:00 edit format to an integer, with forgiveness for missing 0's
	if(s.UpperCase() == "HOLD"){
		return 0;
	}
	for(int i=1;i<=s.Length();i++) if(s[i]==' ') s[i]='0';
	int pos = s.Pos(":");
	AnsiString hr;
	hr = s.SubString(1,pos-1);
	if(hr == NULL){
		hr = "0";
	}
	hr.Trim();
	if (hr==" " || hr == NULL){
		hr="0";
	}
	AnsiString min;
	min = s.SubString(pos+1,2);
	if(min == " " || min == NULL){
		min = "0";
	}
	min.Trim();
	if (min==" ") min="0";
	int i = hr.ToIntDef(0)*3600+min.ToIntDef(0)*60; //i holds number of seconds desired
	return i;
}
//---------------------------------------------------------------------------
AnsiString	__fastcall Utility::PadTo00000(AnsiString s)
{/// Make sure s is 5 characters long by padding left end with zeroes e.g. 12 -> 00012
	if(s.Length() > 5){    																											/// Trim to the first 5 if longer than 5
		s = s.SubString(1,5);
	}
	while(s.Length() < 5){																											/// Pad to 5 if shorter
		s = "0"+s;
	}
	return s;
}
//---------------------------------------------------------------------------
AnsiString  __fastcall Utility::SecondsToHHHMM(int seconds)
{/// Convert seconds into HHHMM string XLA wants
	return IntToHHHMM(seconds);
}
//---------------------------------------------------------------------------
double __fastcall Utility::HHHMMToSeconds(AnsiString HHHMM)
{/// Returns the number of seconds in an HHHMM (no colon) formatted string
	if(HHHMM.UpperCase() == "HOLD"){
		return 0.0;
	}
	HHHMM = PadTo00000(HHHMM);
	AnsiString hr = HHHMM.SubString(1,3);
	int h = hr.ToIntDef(0);
	AnsiString min = HHHMM.SubString(4,2);
	int m = min.ToIntDef(0);
	double d = h*3600 + m*60;
	return d;
}
//---------------------------------------------------------------------------
int __fastcall Utility::HMSToInt(AnsiString s)
{///Convert from 00:00:00 or 000:00:00 edit format to an integer, with forgiveness for missing 0's
	for(int i=1;i<=s.Length();i++) if(s[i]==' ') s[i]='0';
	int pos = s.Pos(":");
	AnsiString hr = s.SubString(1,pos-1);
	hr.Trim();
	if (hr==" ") hr="0";
	AnsiString min = s.SubString(pos+1,2);
	min.Trim();
	if (min==" ") min="00";
	AnsiString sec=s.SubString(pos+4,2);
	int Interval=3600*hr.ToInt()+60*min.ToInt()+sec.ToInt();
	return Interval;
}
//---------------------------------------------------------------------------
AnsiString __fastcall Utility::IntToHHHMM(int sec)
{ ///Returns a string in HHHMM format (no colon, reqd. by XLA), input is seconds
	int hr = sec/3600;                      ///Number of hours
	sec=sec-3600*hr;                        ///Get the number of seconds remaining
	int min=DoubleToRoundedInt(sec/60);     /// and convert to minutes
	AnsiString s=(AnsiString)hr;            /// Pad so that 0s are added
	while(s.Length() <= 2) s="0"+s;
	AnsiString t=(AnsiString)min;           ///Format for display
	while(t.Length() <= 1) t="0"+t;
	s=s+t;
	return s;
}
//---------------------------------------------------------------------------
AnsiString __fastcall Utility::IntToHM(int sec)
{///Returns a string in HHH:MM format, input is seconds
	int hr = sec/3600;                      ///Number of hours
	sec=sec-3600*hr;                        ///Get the number of seconds remaining
	int min=DoubleToRoundedInt(sec/60);     /// and convert to minutes
	AnsiString s=(AnsiString)hr;
	if(s.Length() <2) s="0"+s;
	AnsiString t=(AnsiString)min;           ///Format for display
	if(t.Length() <2) t="0"+t;
	s=s+":"+t;
	return s;
}
//---------------------------------------------------------------------------
AnsiString __fastcall Utility::IntToHMS(int t)
{///Convert time in seconds to HH:MM:SS string
	AnsiString sTime;
	unsigned int r=(unsigned int)t;     //Make sure we deal with + numbers
	int hr = r/3600;                    //Number of hours
	r=r-3600*hr;                        //Get the number of seconds remaining
	int min=r/60;                       // and convert to minutes
	int sec=r-60*min;                   //Get seconds
 //Format for display
	sTime=(AnsiString)hr;
	if(sTime.Length() <2) sTime="0"+sTime;
	AnsiString tt=(AnsiString)min;
	if(tt.Length() <2) tt="0"+tt;
	sTime=sTime+":"+tt;
	tt=(AnsiString)sec;
	if(tt.Length() <2) tt="0"+tt;
	sTime=sTime+":"+tt;
	return sTime;
}
//===========================================================================
//Form functions
//---------------------------------------------------------------------------
TComponent * __fastcall Utility::GetComponent(TForm * f, AnsiString name)
{///Loop over all components on the form looking for the one with the matching name
	for (int i = 0; i < f->ComponentCount; i++)
		if ((f->Components[i]->Name == name)) return f->Components[i];
	return NULL;    //Didn't find one!
}
//---------------------------------------------------------------------------
void __fastcall Utility::SetYMinMax(TChart * chart,double LocalYMax,double LocalYMin)
{///Sets the Ymin & Ymax if the values exceed a range... keeps y axis from "bouncing"
	bool bC;
	bC=false;
	double ChartYMax;                       //Allow the intensity display to have fixed y axis. Prevents flicker
	double ChartYMin;
	ChartYMax=chart->LeftAxis->Maximum;    //Set these to the current values for the chart
	ChartYMin=chart->LeftAxis->Minimum;
	if((LocalYMax > ChartYMax-100) || (LocalYMax+300 < ChartYMax))
		{
			ChartYMax=LocalYMax+200;
			bC=true;
		}
	if((ChartYMin > LocalYMin-100) || (ChartYMin+200 < LocalYMin))
		{
			ChartYMin=LocalYMin-200;
			bC=true;
		}
	if(ChartYMin >= ChartYMax)
		{
			ChartYMin=0;
			ChartYMax=50;
		}
	if(bC) chart->LeftAxis->SetMinMax(ChartYMin,ChartYMax);
}
//---------------------------------------------------------------------------
void __fastcall Utility::ClearAllSeries(TChart * chart)
{///Clear all chart series from a TeeChart object
	chart->Enabled=false;
	for(int i=0;i<chart->SeriesCount();i++)
		chart->Series[i]->Clear();
	chart->Enabled=true;
}
//---------------------------------------------------------------------------
bool __fastcall Utility::FormExists(AnsiString name)
{/// Returns true if a form with the name already is on the screen
	for(int i = 0;i< Screen->CustomFormCount;++i){
		if(Screen->CustomForms[i]->Name == name){
		 return true;
		}
	}
	return false;
}
//---------------------------------------------------------------------------
bool __fastcall Utility::CanShowForm(TForm * form)
{/// Returns true and brings the form to the front if the form exists, otherwise returns false
	if(form == NULL){
		return false;
	}
	if(FormExists(form->Name)){
		form->BringToFront();
		form->Show();
		return true;
	}
	return false;
}
//---------------------------------------------------------------------------
AnsiString  __fastcall Utility::GetCurrentVersion()
{///Returns the Product Version string for the current executable
	VerInfo * VersionData;
	VersionData = new VerInfo();
	AnsiString AOSVersion=VersionData->ProductVersion;                        ///Software version
	delete VersionData;
	return AOSVersion;
}
//===========================================================================
///Comm port functions
//---------------------------------------------------------------------------
void __fastcall Utility::GetAvailableCommPorts(TStringList * strngs)
{///Fills the stringlist with the names of the available com ports
	_COMMCONFIG pCC;
	DWORD pDW ;
	strngs->Clear();
	for(int i= 1;i <10;++i)
		{
		 AnsiString sPort;
		 sPort = "COM" + IntToStr(i);
		 if (GetDefaultCommConfig(sPort.c_str(), &pCC, &pDW))
				strngs->Add(sPort);
		}
}
//===========================================================================
///Miscellaneous functions
//---------------------------------------------------------------------------
AnsiString __fastcall Utility::BoolToString(bool b)
{///Returns "false" or  "true" depending on the state of b
	if(b) return "true";
	return "false";
}
//---------------------------------------------------------------------------
AnsiString __fastcall Utility::DBPath()
{/// Returns the local AOS database path executable's location\DB
	AnsiString fp=ExtractFilePath(Application->ExeName);
	fp = fp+"DB\\";
	if(!DirectoryExists(fp))
		ForceDirectories(fp);
	return fp;
}
//---------------------------------------------------------------------------
AnsiString __fastcall Utility::XMLPath()
{/// Returns the local AOS XML path
	AnsiString fp=ExtractFilePath(Application->ExeName);
	fp = fp+"XML\\";
	if(!DirectoryExists(fp))
		ForceDirectories(fp);
	return fp;
}
//---------------------------------------------------------------------------

