//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#ifndef UtilityH
#define UtilityH
#include "XLStatusBar.h"
#include "FlashBoxFrm.h"
#include <Chart.hpp>
//---------------------------------------------------------------------------
class Utility: public TComponent   //Derive as descendant of TComponent for VCL compatibility
{
private:
public:
    __fastcall Utility(TComponent *Owner);
    __fastcall ~Utility();
 ///Objects owned by the Utility class
  TXLStatusBar  * MainStatusBar;                                                         ///Write messages on the MainForm status bar
  TFlashBox     * FlashBox;                                                                ///Present an urgent message to the user
 ///Simple math functions
  void   __fastcall AverageScans(int numvalues,long double * values, long double * average,long double * stdev);
  double __fastcall SumEm(int numvalues,unsigned int * values);
  double __fastcall SumEm(int numvalues,double * values);
  bool   __fastcall Delay(unsigned wait);                                        ///Hold all processing for xxx milliseconds
  double __fastcall RoundTo(double value, int sigdigits);                        ///Round a number to a fixed number of significant digits
  int    __fastcall DoubleToRoundedInt(double d);                                ///Convert a double to an integer, rounding up if decimal > 5
 ///Smoothing, derivative, min/max and peak finding functions
  bool   __fastcall BoxCar(double * X,int boxsize,int num);                      ///Perform a boxcar smoothing on the array
  void   __fastcall RevSmooth(double* x, double* y, int n);
  bool   __fastcall CalcDeriv(double *Data, int NPts, double *Derivative, bool bSmooth);
  bool   __fastcall CalcDeriv(double *Data, int NPts, double *Derivative);
  bool   __fastcall FindMinMax(unsigned short *Data, int StartIndx, int EndIndx, int *Mindex, int *Maxdex);
  bool   __fastcall FindMinMax(unsigned short *Data, int StartIndx, int EndIndx, int *Mindex, int *Maxdex, bool bSmooth);
  bool   __fastcall FindPulse(double *Data,int NPts, int PulseWidth,double *bestsum,int *bestindex);                                ///Use simple sum of PulseWidth points slid across IntensityData, sets bestsum and best index to midpoint of pulse. Return true if successful, else returns false
  bool   __fastcall FindPeak(double *Data, int NPts, int direction, int *PeakLocation, int *FWHM);
  bool   __fastcall PositiveEdgeFinder(unsigned short *Data, int StartIndx, int EndIndx, int *index,int PointsInTransform);         ///Find the index of the positive going edge of a pulse
  bool   __fastcall NegativeEdgeFinder(unsigned short *Data, int StartIndx, int EndIndx, int *index,int PointsInTransform);         ///Find the index of the negative going edge of a pulse
 ///Convert between AUC channel numbers and letters
  AnsiString  __fastcall NumberToLetter(int Number);                             ///Returns the letter corresponding to a number A=0, B=1, etc.
  int         __fastcall LetterToNumber(AnsiString s);                           ///Returns the number corresponding to the letter, A=0, B=1, etc.
 ///Validate floating point
  bool        __fastcall CheckValidFloat(AnsiString s);                          ///Returns true if s is a valid floating point number
 ///Overloaded MeanAndStdDeviation can take array of either ints or doubles
  bool        __fastcall MeanAndStdDeviation(int *values1,int numvalues,double *average,double *stdev);
  bool        __fastcall MeanAndStdDeviation(double *values1,int numvalues,double *average,double *stdev);
  bool        __fastcall MeanAndStdDeviation(unsigned short *values1,int start,int end,double *average,double *stdev);
  bool        __fastcall MeanAndStdDeviation(double *values1,int start,int end,double *average,double *stdev);
 /// Overloaded Mean function to compute average of double array.
  double          __fastcall Mean(double *values, int StartIndex, int EndIndex);
  int             __fastcall Mean(int *values, int StartIndex, int EndIndex);
  unsigned short  __fastcall Mean(unsigned short *values, int StartIndex, int EndIndex);
  double          __fastcall Mean(double *values, int numvalues);
  int             __fastcall Mean(int *values, int numvalues);
	unsigned short  __fastcall Mean(unsigned short *values, int numvalues);
 ///Subroutine timing varialbles and functions, calibrated for function calls
	double StartTime;
	double LastTime;
	double EndTime;
	double CalibrationTime;
	double AccumulatedTime;
	double NumberOfPases;
	void        __fastcall FirstTime();
	void        __fastcall ResetTime();
	void        __fastcall NextTime();
	double      __fastcall GetTimeMilliSeconds();                                   			/// Returns current time in ms
	double      __fastcall GetTimeSeconds(){return (GetTimeMilliSeconds()/1000.);}  			/// Returns current time in seconds
 ///Time formatting functions-
	int         __fastcall HMSToInt(AnsiString s);          															/// Convert from 00:00:00 or 000:00:00 edit format to an integer, with forgiveness for missing 0's
	int         __fastcall HMToInt(AnsiString s);           															/// Convert from 00:00 or 000:00 edit format to an integer, with forgiveness for missing 0's
	AnsiString	__fastcall PadTo00000(AnsiString s);																			/// Make sure s is 5 characters long by padding left end with zeroes
	AnsiString  __fastcall SecondsToHHHMM(int seconds);																		/// Convert seconds into HHHMM string XLA wants
	double 			__fastcall HHHMMToSeconds(AnsiString HHHMM);															/// Returns the number of seconds in an HHHMM (no colon) formatted string
	AnsiString 	__fastcall IntToHHHMM(int sec); 																					/// Returns a string in HHHMM format (no colon, reqd. by XLA), input is seconds
	AnsiString  __fastcall IntToHM(int sec);                                        			/// Convert time in seconds to HH:MM string
	AnsiString  __fastcall IntToHMS(int t);                                         			/// Convert time in seconds to HH:MM:SS string
	int         __fastcall TimeToInt(AnsiString s);                                 			/// Convert either 00:00 or 00:00:00 to seconds
	AnsiString  __fastcall SetScanNumOrTime(int value);                             			/// Returns the text of the box to either number of scans (if value > 0) or time to scan (value < 0)
	double			__fastcall RpmToW(int rpm);																								/// Convert rpm to omega square
	double			__fastcall RpmToW2(int rpm);																							/// Convert rpm to omega square
 ///Filename functions
	bool        __fastcall IsFloFile(AnsiString filename);                          			///Returns true if the file has .FI* extension
	AnsiString  __fastcall GetUniqueFileName(AnsiString filename);                  			///Check that there is no existing file with this name. If there is, create a unique name
 ///Form functions
	TComponent* __fastcall GetComponent(TForm * f,AnsiString name);                				///Returns the component on the form having the name
	void        __fastcall ClearAllSeries(TChart * chart);                         				///Clear all chart series from a TeeChart object
	void        __fastcall SetYMinMax(TChart * chart,double LocalYMax,double LocalYMin);  ///Sets the Ymin & Ymax if the values exceed a range... keeps y axis from "bouncing"
	bool				__fastcall FormExists(AnsiString name);																		/// Returns true if a form with the name already is on the screen
	bool				__fastcall CanShowForm(TForm * form);																			/// Returns true and brings the form to the front if the form exists, otherwise returns false
 ///Version information
	AnsiString  __fastcall GetCurrentVersion();               														///Returns the Product Version string for the current executable
 ///Comm port functions
	void		 		__fastcall GetAvailableCommPorts(TStringList * strngs);										///Fills the stringlist with the names of the available com ports
 ///Miscellaneous functions
	AnsiString __fastcall BoolToString(bool b);																			///Returns "false" or  "true" depending on the state of b
	AnsiString __fastcall DBPath();																									/// Returns the local AOS database path executable's location\DB
	AnsiString __fastcall XMLPath();																								/// Returns the local AOS XML path
};
extern Utility *Util;

#endif
