//---------------------------------------------------------------------------


#pragma hdrstop

#include "VacuumSystem.h"
#include <math.h>
#include "Utility.h"
#include "Machine.h"
//---------------------------------------------------------------------------
__fastcall TVacuumSystem::TVacuumSystem(TComponent *Owner):TComponent(Owner)
{///The Vacuum system has a vacuum gauge
	StartOfPumping    =0;                   ///When was the pump first started
	PumpTime          =0;                   ///For the simulation, how long has it been pumping
	bPumpDown         =false;               ///Machine is in the pumpdown phase
	bVacuumOK         =false;               ///Machine is at a pressure < 50 microns
}
//---------------------------------------------------------------------------
__fastcall TVacuumSystem::~TVacuumSystem()
{
}
//---------------------------------------------------------------------------
void __fastcall TVacuumSystem::StartPumping()
{/// Set start of pumping time & flag
	StartOfPumping=Util->GetTimeSeconds();
	bPumpDown=true;
}
//---------------------------------------------------------------------------
void __fastcall TVacuumSystem::SimulatePumping()
{
	if((bPumpDown
		|| PumpTime < 20)
		&& StartOfPumping !=0){
			PumpTime=(double)Util->GetTimeSeconds() - StartOfPumping;
	}
	if(PumpTime >= 20){///Set it for 0
		bPumpDown = false;												/// No longer in pump down phase
		Machine->XLActual->putVac("50");
		return;
	}
}
//---------------------------------------------------------------------------
AnsiString __fastcall  TVacuumSystem::GetVacuumStatus()
{/// Returns string with current vacuum status
	if(bVacuumOK || Machine->XLActual->getVacInt() < 100){																											/// Handle situation where vacuum is < 50 um
		return "Vacuum OK";
	}
	else{																																/// Vacuum system either pumping or not puming
		if(bPumpDown){																										/// Vacuum pumps are on
			double VacWait=(double)Util->GetTimeSeconds()- StartOfPumping;  ///Get time in seconds since pumping started
			return "Pumping down: "+Util->IntToHMS(VacWait);
		}
		else{
			return "Pumps off";
		}
	}
}
//---------------------------------------------------------------------------
#pragma package(smart_init)
