#include <vcl.h>
#pragma hdrstop
//---------------------------------------------------------------------------


#pragma hdrstop

#include "XLCommIn.h"
#include <stdio.h>
//#include "DataFileManager.h"
#include "XLScanControl.h"

//---------------------------------------------------------------------
//         Routines to handle messages from XL
//---------------------------------------------------------------------
__fastcall XLCommIn::XLCommIn(TComponent *Owner):XLCommOut(Owner)
{
  MsgSize = 0;                                                                ///Number of bytes in the inmsg
	XLActual     =new XLData(this);                                             ///Holds the actual XL rpm, time, temp settings
	XLActual->XMLDoc = "MachineActual";																					/// The name of the XMLDocument containing the settings
	OpticsActual = new XLHWData(this,true);                                     ///Holds the current settings for the XLA optics
	LastRpm			 = 0;
}
//---------------------------------------------------------------------------
__fastcall XLCommIn::~XLCommIn()
{
  if(XLActual)    delete XLActual;
  if(OpticsActual) delete OpticsActual;
}
//---------------------------------------------------------------------------
void __fastcall XLCommIn::MessageDecoder(char* inmsg,int msgsize)
{/// Decode the received characters and write data to XLdata
 ///Note: XLReadThread has already determined that we have a valid
 /// message by checking for STX and ETX characters in RxBuffer[0].
	AnsiString s;
  UINT diagNum;
  MsgSize = msgsize;                                                          ///How many characters are in inmsg
  if(MsgStatus) MsgStatus->AddTo(inmsg);
  if(VerifyCheckSum(inmsg))
  {///Make sure the message has a proper checksum
	  switch (inmsg[1])
      {
        case FieldService:
				StoreFS(inmsg);
				break;
        case DATA:  	  	        ///Data are in the inmsg
          switch (inmsg[2])
            {///Tell what kind of data we are dealing with
              case '1':          ///These are various XL error messages
              case '2':
              case '3':
              case '4':
              case '5':
              case '6':
              case '7':
              case '8':
              case '9':
                diagNum = (inmsg[2]-48)*10 + (inmsg[3]-48);
                DisplayError(FUGE_DIAGNOSTIC,diagNum);
                break;
              case 'X':
                s="XLCommIn: Data requested, but no data available";
								MachineDB->AddError(1000,s);																	///Use default severity and action
                break;
              default:
                StoreData (inmsg);
                break;
            }
    	    break;
      case DAB_ERROR:
  	  	DecodeDABError(inmsg);
    	  break;
      case 'S':
        switch (inmsg[2])
          {///Sending status, find out what kind
            case XLStatus:
              switch (inmsg[3])
                {
                  case 'A':
                  case 'D':
                  case 'S':
                  case 'R':
                    if(LogStatus)
                      {
                        s="Machine Status: "+(AnsiString)inmsg;
												MachineDB->AddError(1001,s);
                      }
                    StoreStatus(inmsg);
                    break;
                  default:
                    break;
                }
              break;
            case XLSettings:
              StoreSettings(inmsg);
              break;
            default:
              //s="Machine: Unrecognized XL message:"+(AnsiString)inmsg;
							//ErrorLog->Add(s);
              break;
          }
        break;
      case REQUEST:      	    //Send message to user
        XLRequestMessage(inmsg);
      	break;
	    case 'I':
				StoreElapsedTime(inmsg);
    	  break;
      case 'C':               //Calibration scan message
        // Not handling this at the moment
        break;
			case 'V':								// Vacuum at 50 microns
      	XLActual->putVacInt(1);
			break;
			case 'Z':
        // Not handling this at the moment
        //s="Unrecognized XL message:"+(AnsiString)inmsg;
				//ErrorLog->Add(s);
        break;
      case '1':
      case '2':
      case '3':
      case '4':
      case '5':
      case '6':
      case '7':
      case '8':
      	//DecodeScanStatus( inmsg[2] );
        //s="Machine: Unrecognized XL message:"+(AnsiString)inmsg;
        //ErrorLog->Add(s);
        break;
	    default:
        //s="Machine: Unrecognized XL message:"+(AnsiString)inmsg;
        // ErrorLog->Add(s);
    	  break;
	  }
  }
}
//---------------------------------------------------------------------
void XLCommIn::StoreData(char * inmsg)
{
  switch (inmsg[2])
  {
			case 'D':				// delay scan data
			case 'T':				// time scan data; same as delay scan data???...
//				SaveFsScanData (firstDataByte, etxPos);
				XLScanController->putFsScanActive (false);
				XLScanController->putScansFinished (true);
				XLScanController->XLScanner->putScanCondition ("not scanning");
				XLScanController->putXlaIdle (true);		// for scan control
				break;
			case 'R':				// radial or wavelength scan data
			case 'W':
			case 'C':				// wavlength calibration data
				SaveScanData(inmsg);
				break;
			default:
				MachineDB->AddError(1002,"XLCommIn: Unknown scan data scan type");
				break;
		}
}
//---------------------------------------------------------------------
bool XLCommIn::InitXLDataFile(char * inmsg,XLDataFile * xldf)
{///Create a Beckman style header line 2 from the inmsg
	bool ret = false;                                 ///Set a flag if this doesn't work
	char data [80];                                   ///Gather up the information
  char dataType;                                    ///What sort of data are we dealing with
	float position;                                   ///For wavelength scan, where were these data acquired
  float temp;                                       ///Temperature
	long time;                                        ///Number of seconds since exeriment started
	double wavelength;                                ///For a radial scan, what was the wavelength
	char speed [6];                                   ///Rpm
 ///Gather up the datadefinition information and put it into the file
  short cellNum = inmsg[3] - 48 - 1;                ///Good god, convert the ascii character to a number, less one...
	XLScanController->DataCellNum (cellNum);          ///Set the scan controller
  temp = getTemp(inmsg);
  time = getTime(inmsg);
  strcpy (speed, XLActual->getSpeed());
  int replicates = atoi (XLScanController->XLScanner->getReplicates (cellNum));
  if(inmsg[2] == 'R') position = getWave(inmsg);
  else position = getRadius(inmsg);
  switch (inmsg[2])
    {///Handle data for different scan types
      case 'W':
      case 'C':			// wavelength calibration data
        if(inmsg[1] == 'O') dataType = 'W';								// OD
        else                dataType = 'V';               // intensities
        sprintf (data, "%c %c %04.1f %05s %07ld%7.4fE%02d %5.3f %-2d",  // 40 bytes with /n
                  dataType, 						// data type
                  inmsg [1], 				    // cell #
                  temp,									// temperature
                  speed,								// speed
                  time,									// time
                  getW2tMantissa(inmsg),// w2t
                  getW2tExponent(inmsg),// w2t
                  position,							// radius or wavelength depending on scan type
                  replicates
            );                          // end sprintf
        break;
      case 'R':
        switch (inmsg[1])
          {
            case 'F':			                  // field service scan
              dataType = 'I';               // intensities
              sprintf (data, "%c %c %04.1f %05s %03.0f", 				// 19 bytes with /n
                    dataType, 						// data type
                    inmsg [2], 				    // cell #
                    temp,									// temperature
                    speed,								// speed
                    position							// radius or wavelength depending on scan type
                );                        //end sprintf
              break;
            default:			// non-field service scan
              if(inmsg[1] == 'O') dataType = 'R';								// OD
              else dataType = 'I';              // intensities
              sprintf (data, "%c %c %04.1f %05s %07ld%7.4fE%02d %03.0f %-2d",  // 38 bytes w/ /n
                    dataType, 						// data type
                    inmsg [3], 				    // cell #
                    temp,									// temperature
                    speed,								// speed
                    time,									// time
                    getW2tMantissa(inmsg),// w2t
                    getW2tExponent(inmsg),// w2t
                    position,							// radius or wavelength depending on scan type
                    replicates
                );                        //end sprintf
              break;
          }
      break;
  }
  xldf->SetHeader2((AnsiString)data);
  return ret;
}
//---------------------------------------------------------------------
bool __fastcall XLCommIn::SaveScanData(char * inmsg)
{///Format the data and save it as an XLDataFile for further manipulation
 //	char drive [MAXDRIVE], dir [MAXDIR], file [MAXFILE], ext [MAXEXT], name [MAXPATH];
 //	char comment [COMMENTLEN], lastLine [9];	// comment needs to be defined in scandata.h
	char data [80];
  char dataType;
	float position;
  float temp;
	long time;
	double wavelength;
	char speed [6];
	bool failure = false;
	short cellNum = inmsg[3] - 48 - 1;                          ///Good god, convert the ascii character to a number, less one...
	XLScanController->DataCellNum (cellNum);
  XLDataFile * xldf = new XLDataFile(this);
  InitXLDataFile(inmsg,xldf);                                 ///Create the Header2 data, and fill out the XLDataFile
 ///AOS attempt to create XLA data file
  double x      = 0;                                          ///x value for the XLDataFile- may be r or wavelength
  double y      = 0;                                          ///y value for the XLDataFile- may be absorbance, intensity
  double z      = 0;                                          ///z value for hte XLDataFile- may be intensity or std dev
  double inc    = 0;                                          ///Field service scan- incident light detector intensity
  double pmt    = 0;                                          ///Field service scan- pmt voltage
  double pga    = 0;                                          ///Field service scan- PGAGain
  double sdelay = 0;                                          ///Field service scan- sample sector flash lamp delay
  double rdelay = 0;                                          ///Field service scan- reference sector flash lamp delay
  int index = 0;
  do {///Now fill the array
      if(inmsg [2] == 'R') x = getRadius (inmsg,index);		  ///Radial scan, x values are radii
      else                 x = getWave (inmsg,index);	      ///Wavelength scan, x values are wavelengths
			switch (inmsg [3])                                      ///Now determine whether we have an OD or intensity
         {///Want to know if it is an absorbance 'O' or Intensity file 'R' or 'I'
          case 'O':			// OD data; 40 bytes with /n
            y = getOd(inmsg,index);
            z = getStdDev(inmsg,index);
            break;
          case 'R': 		// raw intensity data; 40 bytes with /n
          case 'I':
            y = getRefSig(inmsg,index);                          ///The y value in an intensity scan is the reference signal
            if (inmsg[5] != 'C') z =getSampleSig(inmsg,index);   ///If not a wavelength calibration scan, get the other sector intensity
            break;
          case 'F':			// field service data; 95 bytes with /n
            y       = getRefSig(inmsg,index);
            z       = getSampleSig(inmsg,index);
            inc     = getIncSig(inmsg,index) * 1.0e-04;
            pmt     = getPmtV(inmsg,index);
            pga     = getPgaGain(inmsg,index);
            sdelay  = getSampDelay(inmsg,index) * 1.0e-02;
            rdelay  = getRefDelay(inmsg,index) * 1.0e-02;
            break;
          default :
            DisplayError (SCAN_FILE_ERROR,index);
            failure = true;
            break;
         }
			index++;
		} while (!CheckEndMsg(inmsg,index) || failure);
  delete xldf;                                                                ///Done with the XLDataFile object, so delete it
  if (!failure)
    {// data has been saved; save the last scan's name
      XLScanController->putLastScanName(XLScanController->XLScanner->getFilePath());
      if ((inmsg[2] == 'C') || XLScanController->GetDispData())
        {///Calibration scan
          XLScanController->XLScanner->putScanCondition ("not scanning");
          XLScanController->putXlaIdle (true);		                            ///Flag that we are in an idle state for scan control
        }
    }
 return !failure;                                                             ///Returns true if successful
}
//---------------------------------------------------------------------
bool __fastcall XLCommIn::CheckEndMsg(char * inmsg, int index)
{///Returns true if there is no more data in inmsg
  if((getWave (inmsg,index) != '\0') && (inmsg[2] != 'R')
    || ((getRadius (inmsg,index) != '\0') && (inmsg[2] == 'R')))
    return true;
  else return false;
}
//---------------------------------------------------------------------
void XLCommIn::DecodeDABError (char* inmsg)
{///There is an error being reported by the XL data acquisition board
	int errNum = 0;
	char err [80];
  errNum = (inmsg[1]-48)*10 + (inmsg[2]-48);
	switch (errNum){
		case 1:
			strcpy(err, "Wavelength out of range.");
			break;
		case 2:
			strcpy(err, "UART error.");
			break;
		case 3:
			strcpy(err, "Initializing calibration parameters.");
			break;
		case 4:
			strcpy(err, "Radius out of range.");
			break;
		case 5:
			strcpy(err, "Wavelength eccentricity fit is zero.");
			break;
		case 6:
			strcpy(err, "RTE not running.");
			break;
		case 7:
			strcpy(err, "Incident ranging failure.");
			break;
		case 8:
			strcpy(err, "Baseline ranging failure.");
			break;
		case 9:
			strcpy(err, "Radial rate failure.");
			break;
		case 10:
			strcpy(err, "Wavelength rate failure.");
			break;
		case 11:
			strcpy(err, "Half-wavelength data not found.");
			break;
		case 12:
			strcpy(err, "Massaged data not found.");
			break;
		case 13:
			strcpy(err, "Delay calibration failure.");
			break;
		case 14:
			strcpy(err, "Wavelength calibration failure.");
			break;
		case 15:
			strcpy(err, "Radial calibration failure.");
			break;
		case 16:
			strcpy(err, "Baseline calibration failure.");
			break;
		case 17:
			strcpy(err, "A/D calibration failure.");
			break;
		case 18:
			strcpy(err, "Excessive A/D noise.");
			break;
		case 19:
			strcpy(err, "Radial servo timeout.");
			break;
		case 20:
			strcpy(err, "Wavelength servo timeout.");
			break;
		case 21:
			strcpy(err, "Single pt. wavelength calibration failure.");
			break;
		case 22:
			strcpy(err, "Radial A/D ranging failure.");
			break;
		case 23:
			strcpy(err, "Wavelength A/D ranging failure.");
			break;
		case 24:
			strcpy(err, "RAM checksum failure.");
			break;
		case 25:
			strcpy(err, "Servo not stable.");
			break;
		case 26:
			strcpy(err, "Stray light failure.");
			break;
		case 27:
			strcpy(err, "Raw data buffer full.");
			break;
		case 28:
			strcpy(err, "Output buffer full.");
			break;
		case 29:
			strcpy(err, "Peak not found.");
			break;
		case 30:
			sprintf(err, "Illegal PC->DAB command (%s).", inmsg);
			break;
		case 31:
			strcpy(err, "Input failure.");
			break;
		case 32:
			strcpy(err, "Checksum error.");
			break;
		case 33:
			strcpy(err, "Noisy data.");
			break;
		case 34:
			strcpy(err, "No output type failure.");
			break;
		case 35:
			strcpy(err, "Wavelength down slope not found.");
			break;
		case 36:
			strcpy(err, "Invalid reference data.");
			break;
		case 37:
			strcpy(err, "Coprocessor not ready.");
			break;
		case 38:
			strcpy(err, "Coprocessor OD out of range.");
			break;
		case 39:
			strcpy(err, "Calibration RAM checksum failure.");
			break;
		case 40:
			strcpy(err, "Calibration RAM check failure.");
			break;
		case 41:
			strcpy(err, "Out of RAM space.");
			break;
		case 42:
			strcpy(err, "Output buffer out of range.");
			break;
		case 43:
			strcpy(err, "RAM buffer too large.");
			break;
		case 44:
			strcpy(err, "STX not found.");
			break;
		case 45:
			strcpy(err, "ETX not found.");
			break;
		case 46:
			strcpy(err, "Could not generate checksum.");
			break;
		case 47:
			strcpy(err, "No displayable data available.");
			break;
		case 48:
			strcpy(err, "Period counter failure.");
			break;
		case 49:
			strcpy(err, "Incident count too low.");
			break;
		case 50:
			strcpy(err, "Incident count too high.");
			break;
		case 51:
			strcpy(err, "Excessive stray light.");
			break;
		case 52:
			strcpy(err, "Saturated baseline data.");
			break;
		case 53:
			strcpy(err, "Intensity too high.");
			break;
		case 54:
			strcpy(err, "Intensity too low.");
			break;
		case 69:
			strcpy(err, "DAB->XL com failure");
			break;
		case 70:
			strcpy(err, "XL com failure");
			break;
		case 213:
			// Think this occurs when DP has previously been stopped from program
			break;
		default:
//			sprintf(err, "Unknown error #%d received.", errNum);
			break;
	}
	AnsiString s="Machine: "+(AnsiString)err;
	MachineDB->AddError(errno,s);
}
//---------------------------------------------------------------------
void XLCommIn::StoreStatus(char* inmsg)
{//Get current machine status (jba 08/12/02)
	char temp[8];
	XLActual->putXlStatus(inmsg[3]);
	///Get current rotor speed
	/// Save the old rotor speed for calculations of how fast the speed is changing
	LastRpm = XLActual->getSpeedInt();
	temp[0]=inmsg[4];
	temp[1]=inmsg[5];
	temp[2]=inmsg[6];
	temp[3] = '0';
	temp[4] = '0';
	temp[5] = '\0';
	XLActual->putSpeed(temp);
	//Deal with zero rotor speed after a shutdown
	if (atoi(temp) == 0) XLActual->ResetRunTime();
	//Get current run time
	temp[0]=inmsg[7];
	temp[1]=inmsg[8];
	temp[2]=inmsg[9];
	temp[3]=inmsg[10];
	temp[4]=inmsg[11];
	temp[5] = '\0';
	XLActual->putTime(temp);
	//Save the current temperature
	temp[0]=inmsg[12];
	temp[1]=inmsg[13];
	temp[2]=inmsg[14];
	temp[3] = '\0';
	XLActual->putTemp(temp);
	//Save the current w2t
	temp[0]=inmsg[15];
	temp[1]=inmsg[16];
	temp[2]=inmsg[17];
	temp[3]=inmsg[18];
	temp[4]=inmsg[19];
	temp[5] = '\0';
	XLActual->putW2t(temp);
	//What does the instrument think the acceleration is
	temp[0]=inmsg[20];
	temp[1] = '\0';
	XLActual->putAccel(temp);
	//Oh, and what does the machine think the braking is
	temp[0]=inmsg[21];
	temp[1] = '\0';
	XLActual->putDecel(temp);
	XLActual->ActualsObtained(true);
}
//---------------------------------------------------------------------
void XLCommIn::StoreSettings(char* inmsg)
{///Desired settings being returned
	char temp[8];
 //Retrieve the Set speed
	temp[0]=inmsg[3];
	temp[1]=inmsg[4];
	temp[2]=inmsg[5];
	temp[3]=inmsg[6];
	temp[4]= '0';
	temp[5]= '\0';
	XLDesired->putSpeed(temp);
 //Retrieve the Set time
	temp[0]=inmsg[7];
	temp[1]=inmsg[8];
	temp[2]=':';
	temp[3]=inmsg[9];
	temp[4]=inmsg[10];
	temp[5] = '\0';
	if (strcmp(temp, "00:00") == 0) strcpy(temp, "hold");
	XLDesired->putTime(temp);
 //Retrieve the set Temperature
	temp[0]=inmsg[11];
	temp[1]=inmsg[12];
  //	temp[2]=inmsg[13];
  temp[2] = '0';  // Ignore last digit from XL
	temp[3] = '\0';
	XLDesired->putTemp(temp);
 //Retrieve w2t
  temp[0]=inmsg[14];
  temp[1]='.';
  temp[2]=inmsg[15];
  temp[3]=inmsg[16];
  temp[4]='E';
  temp[5]=inmsg[17];
  temp[6]=inmsg[18];
  temp[7] = '\0';
	if (temp[0] == 'X') strcpy(temp, "0");
	XLDesired->putW2t(temp);
 //Get the acceleration setting
	temp[0]=inmsg[19];
	temp[1] = '\0';
	XLDesired->putAccel(temp);
 //Get the braking setting
	temp[0]=inmsg[20];
	temp[1] = '\0';
	XLDesired->putDecel(temp);
  XLDesired->SettingsObtained(true);
  XLDesired->Changed=true;
}
//---------------------------------------------------------------------
bool __fastcall XLCommIn::StoreCalibration (char * inmsg)
{
  bool ret = true;
 	char calData [80], temp [10];
	int j = 0;
	int i = 1;
	int index;
	while (inmsg[i] != ETX)
   {
		calData [j++] = inmsg[i++];
		if (j >= CALDATA_SIZE)
    {
			MachineDB->AddError(1003,"XLCommIn::StoreCalibration- Cal data overrun.");
			break;
		}
	}
	calData [j - 2] = '\0';
/* Will need to find a way to save a CalParms.Cal file
	if (!cal->writeCalFile (calData))
		xlaLog.Add ("CAL", "Calibration file write error.", wndw->diagDialog);
*/
	// now put the data in the calibration object
	CalSettings->putHiWave (calData + 63);
	calData [63] = '\0';
	CalSettings->putLowWave (calData + 60);
	calData [52] = '\0';	// skip factory 220nm & count at 220nm settings
	CalSettings->putHiLimit (calData + 48);
	calData [48] = '\0';
	sprintf (temp, "%d", atoi (calData + 44));
	CalSettings->putLowLimit (temp);
	calData [44] = '\0';
	CalSettings->putHiCount (calData + 40);
	calData [40] = '\0';
	sprintf (temp, "%d", atoi (calData + 36));
	CalSettings->putLowCount (temp);
	if (atoi (CalSettings->getLowLimit()) == 0) {
		CalSettings->putLowCount ("");
		CalSettings->putLowLimit ("");
	}
	if (atoi (CalSettings->getHiLimit()) == 0) {
		CalSettings->putHiCount ("");
		CalSettings->putHiLimit ("");
	}
	if (calData [35] & 0x0001) {
		CalSettings->putWaveCald ("Calibrated");
		CalSettings->putRecalWave (0);
	}
	else {
		CalSettings->putWaveCald ("Not Calibrated");
		CalSettings->putRecalWave (1);
	}
	calData [31] = '\0';	// skip inside CB hole radius setting
	CalSettings->putOutLimit (calData + 27);
	calData [27] = '\0';
	sprintf (temp, "%d", atoi (calData + 23));
	CalSettings->putInLimit (temp);
	calData [23] = '\0';
	sprintf (temp, "%5.3f", atof (calData + 19) / 1000.0);
	CalSettings->putOutRad (temp);
	calData [19] = '\0';
	sprintf (temp, "%5.3f", atof (calData + 15) / 1000.0);
	CalSettings->putInRad (temp);
	calData [15] = '\0';
	if (atoi (CalSettings->getOutLimit()) == 0) {
		CalSettings->putOutCount ("");
		CalSettings->putOutLimit ("");
	}
	else {
		sprintf (temp, "%d", atoi (calData + 11));
		CalSettings->putOutCount (temp);
	}
	calData [11] = '\0';
	if (atoi (CalSettings->getInLimit()) == 0) {
		CalSettings->putInCount ("");
		CalSettings->putInLimit ("");
	}
	else {
		sprintf (temp, "%d", atoi (calData + 7));
		CalSettings->putInCount (temp);
	}
	if (calData [6] == '1') {
		CalSettings->putRadCald ("Calibrated");
		CalSettings->putRecalRad (0);
	}
	else {
		CalSettings->putRadCald ("Not Calibrated");
		CalSettings->putRecalRad (1);
	}
	calData [6] = '\0';
	sprintf (temp, "%ld", atol (calData + 4) * 1000);
	CalSettings->putDelayCalSpeed (temp);
	calData [4] = '\0';
	sprintf (temp, "%5.2f", atof (calData) / 100.0);
	CalSettings->putCollarOffset (temp);
  return ret;
}
//---------------------------------------------------------------------------
bool __fastcall XLCommIn::StoreFS(char * inmsg)
{///Store the Field service data
  bool ret=true;
	short i;
	char temp [8], buff [8];
	int index = 1;
	int pgaGain = inmsg[index] - 48;
 // PGA gain; 1=1, 2=2, 3=4, 4=8, 5=16, 6=32, 7=64, 8=128, 9=256
	pgaGain = 1 << (pgaGain -1);
	OpticsActual->putPgaGain (itoa (pgaGain, temp, 10));
	index = (index + 1);
	temp [0] = inmsg [index];
	temp [1] = '\0';
	OpticsActual->putIncGain (temp);
	for (i = 0; i < 4; i++)
    {
      index = (index + 1);
      temp [i] = inmsg [index];
    }
	temp [4] = '\0';
	sprintf (buff, "%d", atoi (temp));
	OpticsActual->putPmtVoltage (buff);

	for (i = 0; i < 5; i++) {
		index = (index + 1) % RXQUEUESIZE;
		temp [i] = inmsg[index];
	}
	temp [5] = '\0';
	sprintf (buff, "%ld", atol (temp));
	OpticsActual->putDelayTime (buff);

	for (i = 0; i < 3; i++) {
		index = (index + 1) % RXQUEUESIZE;
		temp [i] = inmsg[index];
	}
	temp [3] = '\0';
	OpticsActual->putWavePos (temp);

	for (i = 0; i < 4; i++) {
		index = (index + 1) % RXQUEUESIZE;
		temp [i] = inmsg[index];
	}
	temp [4] = '\0';
	sprintf (buff, "%5.3f", atof (temp) / 1000.0);
	OpticsActual->putRadPos (buff);

	for (i = 0; i < 5; i++) {
		index = (index + 1) % RXQUEUESIZE;
		temp [i] = inmsg[index];
	}
	temp [5] = '\0';
	sprintf (buff, "%ld", atol (temp));
	OpticsActual->putRawData (buff);

	for (i = 0; i < 3; i++) {
		index = (index + 1) % RXQUEUESIZE;
		temp [i] = inmsg[index];
	}
	temp [3] = '\0';
	sprintf (buff, "%d", atoi (temp));
  //	OpticsActual->putIntTime (buff);
  /*
    !!! implement when DAB changes have been made
    OpticsActual->putStrobeRate());
    OpticsActual->putStrobeCell());
    OpticsActual->putRadSpeed());
    OpticsActual->putWaveSpeed());
    OpticsActual->putIncData());
  */
  return ret;
}
//---------------------------------------------------------------------
void XLCommIn::XLRequestMessage(char* inmsg)
{///A request message has been received
	AnsiString s;
 	switch (inmsg[2])
    {
			case 'C' :			// Request for calibration.
				s = "Z";
				SendXLMessage(s.c_str());
				break;
			default:
				s = "Machine: Unrecognized request";
				MachineDB->AddError(1004,s);
				break;
		}
}
//---------------------------------------------------------------------
void XLCommIn::DisplayError(int error, int index)
{///An XL error has occurred (not DAB, which is handled separately)
	char err [80];
	switch (error)
		{/// What errors do we need to check here?
			case FUGE_DIAGNOSTIC:
				switch (index)
					{
						case 96:
							strcpy(err, "DAB->XL parameter out of range");
							break;
						case 97:
							strcpy(err, "DAB->XL checksum error");
							break;
						case 98:
							strcpy(err, "XL caution condition occurred");
							break;
						case 99:
							strcpy(err, "XL shutdown condition occurred");
							MachineError=true;                              ///Flag the error so that the experiment may be stopped
							break;
						default:
							sprintf(err, "XL diagnostic %d occurred", index);
							break;
					}
				break;
			case COMMCKSUM:
				strcpy(err, "Checksum Error");
				break;
			default:
				sprintf(err, "Unknown diagnostic %d occurred", index);
				break;
		}
	AnsiString s="Machine: "+(AnsiString)err;
	MachineDB->AddError(index,s);
}
//---------------------------------------------------------------------
//							S t o r e E l a p s e d T i m e
// Also store mantissa and exponent of w2t.
//---------------------------------------------------------------------
void XLCommIn::StoreElapsedTime(char* inmsg){
	char temp [8];
	int i;

	for (i = 0; i < 7; i++) temp[i]=inmsg[i+1];
	temp[7] = '\0';
  XLActual->ElapsedTime(temp);

	for (i = 0; i < 6; i++) temp[i]=inmsg[i+8];
	temp[6] = '\0';
  XLActual->Mantissa(atof(temp) * 1.0e-5);

	for (i = 0; i < 2; i++) temp[i]=inmsg[i+13];
	temp[2] = '\0';
  XLActual->Exponent(atoi(temp));
}
//---------------------------------------------------------------------
//									Checksum functions
//---------------------------------------------------------------------
bool XLCommIn::VerifyCheckSum(char* inmsg){
	AnsiString s;
	int sum = 0;
	char chksum[2];
	int length = strlen(inmsg);
	if (length >=5)
		{/// Compute check sum and compare against checksum in inmsg.
			for (int i=0; i<(length-3); i++) sum +=inmsg[i];
			chksum[0] = ((sum / 10) % 10) + 48;
			chksum[1] = sum % 10 + 48;
			if((chksum[0] == inmsg[length-3])
				&& (chksum[1] == inmsg[length-2]))        /// Check sum succeeded!
					return true;
			else                                        /// Check sum failed.
				return false;
		}
	else
		{
			//Error in size of message
			s="Machine: Error Decoding Input Message: Invalid input message length.";
			MachineDB->AddError(1005,s);
			return false;
    }
}
//===========================================================================
//      Crack XLA message for creating a file
//---------------------------------------------------------------------------
float XLCommIn::getRadius(char *inmsg)
{///returns for wavelength scans
  char radius [5];
  int offset = 4;
  for (int i = 0; i < sizeof (radius) - 1; i++)
      radius [i] = inmsg[offset + i];
  radius [sizeof (radius) - 1] = '\0';
  return atof (radius) / 1000.0;
}
//---------------------------------------------------------------------------
float XLCommIn::getRadius (char * inmsg,int index)
{//Returns radius for other types of scans
  char radius [5];
  int offset = 26;
  switch (inmsg[1])
    {
      case 'R':
      case 'I':
        offset += index * 18;
        break;
      case 'O':
        offset += index * 17;
        break;
      case 'F':				// field service scan
        offset = 13;
        offset += index * 44;
        break;
      default :
        DisplayError (SCAN_FILE_ERROR,offset);
        return 0;
    }
  if (inmsg[offset] == '\0')
    return NULL;	 // indicate end of data
  else
    {
      for (int i = 0; i < sizeof (radius) - 1; i++)
        radius [i] = inmsg[offset + i];
      radius [sizeof (radius) - 1] = '\0';
      return atof (radius) / 1000.0;
    }
}
//---------------------------------------------------------------------------
long XLCommIn::getTime(char * inmsg)
{///Return the run time in seconds from the input queue
  int offset;
  char time [8];
  if(inmsg[2] == 'R') offset = 7;
  else offset = 8;
  for (int i = 0; i < sizeof (time) - 1; i++)
    time [i] = inmsg[(offset + i)];
  time [sizeof (time) - 1] = '\0';
  return atol (time);
}
//---------------------------------------------------------------------------
float XLCommIn::getW2tMantissa(char * inmsg)
{///Returns the x.xxx of w2t
  int offset;
  char mantissa [5];
  if(inmsg[2] == 'R') offset = 14;
  else offset = 15;
  for (int i = 0; i < sizeof (mantissa) - 1; i++)
    mantissa [i] = inmsg[offset + i];
  mantissa [sizeof (mantissa) - 1] = '\0';
  return atof (mantissa) / 1000.0;
}
//---------------------------------------------------------------------------
int XLCommIn::getW2tExponent(char* inmsg)
{///Returns the exponent of w2t
  int offset;
  char exponent [3];
  if(inmsg[2] == 'R') offset = 18;
  else  offset = 19;
  for (int i = 0; i < sizeof (exponent) - 1; i++)
    exponent[i] = inmsg[offset + i];
  exponent [sizeof (exponent) - 1] = '\0';
  return atoi (exponent);
}
//---------------------------------------------------------------------------
int XLCommIn::getSpeed(char * inmsg)
{///Returns the rotor speed in thousands
  int offset;
  char speed [4];
  switch (inmsg[1])
    {///Offset for the rotor speed depends on the type of scan
      case 'F':		// field service scan
        offset = 7;
        break;
      default:		// non-field service scan
        if(inmsg[2] == 'R') offset = 20;
        else offset = 21;
        break;
    }
  for (int i = 0; i < sizeof (speed) - 1; i++)
    speed [i] = inmsg[offset + i];
  speed [sizeof (speed) - 1] = '\0';
  return atoi (speed) * 100;
}
//---------------------------------------------------------------------------
float XLCommIn::getTemp(char * inmsg)
{///Returns the temperature as xx.x
  int offset;
  char temp [4];
  switch (inmsg[1])
    {
      case 'F':		// field service scan
        offset = 10;
        break;
      default:		// non-field service scan
        if(inmsg[2] == 'R') offset = 23;
        else offset = 24;
        break;
    }
  for (int i = 0; i < sizeof (temp) - 1; i++)
    temp [i] = inmsg[offset + i];
  temp [sizeof (temp) - 1] = '\0';
  return atof (temp) / 10;
}
//---------------------------------------------------------------------------
float XLCommIn::getWave(char * inmsg)
{///Get wavelength for radial scans
  int offset = 4;
  char wave [4];
  for (int i = 0; i < sizeof (wave) - 1; i++)
    wave [i] = inmsg[offset + i];
  wave [sizeof (wave) - 1] = '\0';
  return atoi (wave);
}
//---------------------------------------------------------------------------
float XLCommIn::getWave(char * inmsg,int index)
{///For wavelength scan... offset calculated here
  int offset = 27;
  char wave [4];
  switch (inmsg[1])
    {
      case 'R':
      case 'I':
        offset += index * 17;
        break;
      case 'O':
        offset += index * 16;
        break;
      case 'F':
        offset = 4;
        offset += index * 16;
        break;
      default :
        DisplayError (SCAN_FILE_ERROR,offset);
        return 0;
    }
  if (inmsg[offset] == '\0')
    return NULL;	 // indicate end of data
  else
    {
      for (int i = 0; i < sizeof (wave) - 1; i++)
        wave [i] = inmsg[offset + i];
      wave [sizeof (wave) - 1] = '\0';
      return atoi (wave);
    }
}
//---------------------------------------------------------------------------
float XLCommIn::getOd(char * inmsg,int index)
{
  int offset = 30;
  char od [8];
  if(inmsg[2] == 'W') offset += index*16;
  else offset += index * 17;
  for (int i = 0; i < sizeof (od) - 1; i++)
    od [i] = inmsg[offset + i];
  od [sizeof (od) - 1] = '\0';
  return atof (od) / 10000.0;
}
//---------------------------------------------------------------------------
float XLCommIn::getStdDev (char * inmsg,int index)
{///Get the standard deviation on the absorbance
  int offset = 37;
  char stdDev [7];
  if(inmsg[2] == 'W') offset += index*16;
  else offset += index * 17;
  for (int i = 0; i < sizeof (stdDev) - 1; i++)
    stdDev [i] = inmsg[offset + i];
  stdDev [sizeof (stdDev) - 1] = '\0';
  return atof (stdDev) / 10000.0;
}
//---------------------------------------------------------------------------
float XLCommIn::getRefSig (char * inmsg,int index)
{///Get the reference intensity
  int offset = 30;
  char refSig [8];
  switch (inmsg[1])
    {
      case 'F':			// field service scan
        offset = 17;
        offset += index * 44;
        break;
      default:			// non-field service scan
        if(inmsg[2] == 'R') offset += index * 18;
        else offset += index * 17;
        break;
    }
  for (int i = 0; i < sizeof (refSig) - 1; i++)
    refSig [i] = inmsg[offset + i];
  refSig [sizeof (refSig) - 1] = '\0';
  return atof (refSig) / 100.0;
}
//---------------------------------------------------------------------------
float XLCommIn::getSampleSig (char* inmsg,int index)
{///Returns the sample intensity
  int offset = 37;
  char sampSig [8];
  switch (inmsg[1])
    {
      case 'F':			// field service scan
        offset = 24;
        offset += index * 44;
        break;
      default:			// non-field service scan
        if(inmsg[2] == 'R') offset += index * 18;
        else offset += index * 17;
        break;
    }
  for (int i = 0; i < sizeof (sampSig) - 1; i++)
    sampSig[i] = inmsg[offset + i];
  sampSig [sizeof (sampSig) - 1] = '\0';
  return atof (sampSig) / 100.0;
}
//---------------------------------------------------------------------------
float XLCommIn::getIncSig (char * inmsg,int index)
{///Returns the incident light signal
  int offset = 31;
  char incSig [8];
  offset += index * 44;
  for (int i = 0; i < sizeof (incSig) - 1; i++)
    incSig[i] = inmsg[offset + i];
  incSig [sizeof (incSig) - 1] = '\0';
  return atof (incSig) * 100.0;
}
//---------------------------------------------------------------------------
int XLCommIn::getPmtV (char * inmsg,int index)
{///What is the voltage on the absorbance PMT
  int offset = 38;
  char pmtV [5];
  offset += index * 44;
  for (int i = 0; i < sizeof (pmtV) - 1; i++)
    pmtV[i] = inmsg[offset + i];
  pmtV [sizeof (pmtV) - 1] = '\0';
  return atoi (pmtV);
}
//---------------------------------------------------------------------------
int XLCommIn::getPgaGain (char * inmsg,UINT index)
{///Returns the setting of the Beckman PGA
  int offset = 42;
  char pgaGain [4];
  offset += index * 44;
  for (int i = 0; i < sizeof (pgaGain) - 1; i++)
    pgaGain [i] = inmsg[offset + i];
  pgaGain [sizeof (pgaGain) - 1] = '\0';
  return atoi (pgaGain);
}
//---------------------------------------------------------------------------
float XLCommIn::getSampDelay (char * inmsg,int index)
{///Get the delay angle for flashing the sample sector
  int offset = 45;
  char sampDelay [7];
  offset += index * 44;
  for (int i = 0; i < sizeof (sampDelay) - 1; i++)
    sampDelay [i] = inmsg [offset + i];
  sampDelay [sizeof (sampDelay) - 1] = '\0';
  return atof (sampDelay);
}
//---------------------------------------------------------------------------
float XLCommIn::getRefDelay (char * inmsg,int index)
{///Return the angle for flashing the reference sector
  int offset = 51;
  char refDelay [7];
  offset += index * 44;
  for (int i = 0; i < sizeof (refDelay) - 1; i++)
    refDelay [i] = inmsg[offset + i];
  refDelay [sizeof (refDelay) - 1] = '\0';
  return atof (refDelay);
}
//---------------------------------------------------------------------------
float XLCommIn::getDelaySig (char * inmsg,int index)
{///Get the delay time
  int offset = 16;
  char refSig [8];
  offset += index * 12;
  for (int i = 0; i < sizeof (refSig) - 1; i++)
    refSig [i] = inmsg[offset + i];
  refSig [sizeof (refSig) - 1] = '\0';
  return atof (refSig) / 100.0;
}
//---------------------------------------------------------------------------
float XLCommIn::getDelayTime (char * inmsg,int index, int etxPos)
{///Returns the time for the delay to some position...
  int offset = 23;
  char delayTime [6];
  offset += index * 12;
  if (inmsg[offset + 7] == ETX)
    return -1.0;		// signals end of data
  else
    {
      for (int i = 0; i < sizeof (delayTime) - 1; i++)
        delayTime [i] = inmsg [offset + i];
      delayTime [sizeof (delayTime) - 1] = '\0';
      return atof (delayTime);
    }
}

#pragma package(smart_init)
