//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "XLData.h"
#include	<stdio.h>
#include <math.h>
#include "XMLUtilities.h"
#include "Machine.h"
#include "Utility.h"

//---------------------------------------------------------------------------
__fastcall XLData::XLData(TComponent * Owner):TComponent(Owner)
{
	IsConstructed = false;			 // true if the class has completed construction
	mantissa = 0.0;
	exponent = 0;
	elapsedTime = 0L;
	ResetRunTime();
	actualsObtained = false;
	settingsObtained = false;
	PutAccelDecelDefaults();
	putSpeed(DEFAULT_SPD);
	putTime(DEFAULT_TIME);
	putTemp(DEFAULT_TEMP);
	putW2t(DEFAULT_W2T);
	putVac(DEFAULT_VAC);
	putXlStatus(XLRUNNING);
	Changed=true;           //Initially want to update settings or status.
	XMLDoc = "MachineActual";
	IsConstructed = true;
}
//---------------------------------------------------------------------------
void XLData::getAll(XLData* source){
	putSpeed(source->getSpeed());
	putTime(source->getXLTime());
	putTemp(source->getTemp());
	putW2t(source->getW2t());
	putVac(source->getVac());
	putAnAccel(source->getAnAccel());
	putAnDecel(source->getAnDecel());
	putAccel(source->getAccel());
	putDecel(source->getDecel());
	putStatus(source->getStatus());
	putXlStatus(source->getXlStatus());
}
//---------------------------------------------------------------------------
float XLData::Mantissa(){
	return mantissa;
}
//---------------------------------------------------------------------------
void XLData::Mantissa(float val){
	mantissa = val;
}
//---------------------------------------------------------------------------
int XLData::Exponent(){
	return exponent;
}
//---------------------------------------------------------------------------
void XLData::Exponent(int val){
	exponent = val;
}
//---------------------------------------------------------------------------
long XLData::ElapsedTime(){
	return elapsedTime;
}
//---------------------------------------------------------------------------
void XLData::ElapsedTime(char* et){
	elapsedTime = atol(et);
}
//---------------------------------------------------------------------------
long XLData::RunTime(){
	return runTime;
}
//---------------------------------------------------------------------------
void XLData::IncRunTime(){
	runTime++;
}
//---------------------------------------------------------------------------
void XLData::ResetRunTime(){
	runTime = 0L;
}
//---------------------------------------------------------------------------
void XLData::ActualsObtained(bool state){
	actualsObtained = state;
}
//---------------------------------------------------------------------------
bool XLData::ActualsObtained(){
	return actualsObtained;
}
//---------------------------------------------------------------------------
void XLData::SettingsObtained(bool state){
	settingsObtained = state;
}
//---------------------------------------------------------------------------
bool XLData::SettingsObtained(){
	return settingsObtained;
}
//===========================================================================
// Setter functions
//---------------------------------------------------------------------------
void XLData::putSpeed(char* data)
{///Setter for the rpm
	if (strcmp(data, "") == 0) strcpy(speed, DEFAULT_SPD);
	else
		{///real value
			long temp = (atol(data) + 50) / 100 * 100;
			ltoa(temp, speed, 10);
		}
	if(IsConstructed && Machine != NULL){
		Machine->MachineDB->ChangeValue(XMLDoc,"RotorSpeed",(AnsiString)speed);
	}
}
//---------------------------------------------------------------------------
void XLData::putTime(char* data)
{///Want time as HHHMM for the XLA value (time), but want seconds for the db
	if(strcmp(data, "") == 0){
		strcpy(time, DEFAULT_TIME);
	}
	else{
		strcpy(time, data);
	}
	AnsiString s = (AnsiString)time;
	if(IsConstructed && Machine != NULL){																				/// NOTE: Time is stored in seconds in the db
	 if(s.UpperCase() == "HOLD"){
		s = s.UpperCase();
	 }
	 else{      																																/// Time contains a number. Make sure it is in the form HHHMM
			s = Util->PadTo00000(s);																								/// Convert HHHMM seconds
			int i = (int)Util->HHHMMToSeconds(s);
			s	= (AnsiString)i;
	 }
	 Machine->MachineDB->ChangeValue(XMLDoc,"Time",s);
	}
}
//===========================================================================
// Temperature formatting is a mess due to integer used for desired and
// double for actual, plus the fact that zeros must be added/deleted
// to match the XL string requirements.
//---------------------------------------------------------------------------
void XLData::putTemp(double Ta)
{/// Convert to char array safely, then call putTemp(char *)
	///Expect data to be in form xx.x
	if(IsConstructed && Machine != NULL){
		AnsiString s = FormatFloat("#0.0",Ta);
		Machine->MachineDB->ChangeValue(XMLDoc,"Temperature",s);
	}
}
//---------------------------------------------------------------------------
void XLData::putTemp(char* data)
{///Temperature data stored as character array with 3 elements.
 /// When formated will read XXX -> XX.X
 /// Check to make sure data is three characters if not add a zero to end.
	char tempdata[4];
	if(data[1] == '\0'){																								/// Data is single digit e.g. '4'... make element 0 element 1, add '0' to front
		data[1]=data[0];  																								///Rearrange to '04', which eventually will be reformatted to '040'
		data[0]='0';
		data[2]='\0';
	}
	tempdata[0] = data[0];
	tempdata[1] = data[1];
	if (data[2] == '\0'){
		tempdata[2] = '\0';
	}
	else{
		tempdata[2] = data[2];
	}
	tempdata[3] = '\0';
	if(IsConstructed && Machine != NULL){
		double d = atof(tempdata);
		if(d > 40.0){
    	d = d/10;
		}
		AnsiString s = FormatFloat("#0.0",d);
		Machine->MachineDB->ChangeValue(XMLDoc,"Temperature",s);
	}
}
//---------------------------------------------------------------------------
void XLData::putTempInt(int data)
{///Setter for the temperature as an integer
	if(IsConstructed && Machine != NULL){
		double d = (double)data;
		AnsiString s = FormatFloat("#0.0",d);
		Machine->MachineDB->ChangeValue(XMLDoc,"Temperature",s);
	}
}
//---------------------------------------------------------------------------
void XLData::putW2t(char* data)
{///Set the omega squared t
	(strcmp(data, "") == 0) ?
		strcpy(w2t, DEFAULT_W2T):
		strcpy(w2t, data);
}
//---------------------------------------------------------------------------
void XLData::SaveW2t(AnsiString s)
{/// W2t saved in the DB separately from w2t from the machine value... program calculates as x.xxxxEyy, machine sends x.xxEyy
	if(IsConstructed && Machine != NULL){
	 Machine->MachineDB->ChangeValue(XMLDoc,"w2t",s);
	}
}
//---------------------------------------------------------------------------
void XLData::putVac(char* data)
{///Set the value of the vacuum
	(strcmp(data, "") == 0) ?
		strcpy(vac, DEFAULT_VAC):
		strcpy(vac, data);
}
//---------------------------------------------------------------------------
void XLData::putVacInt(int data)
{///Set the vacuum as an integer
	itoa(data, vac, 10);
}
//---------------------------------------------------------------------------
void XLData::putAnAccel(char* data)
{///Set the analytical acceleration
	(strcmp(data, "") == 0) ?
		strcpy(anAccel, DEFAULT_ANACCEL):
		strcpy(anAccel, data);
	if(IsConstructed && Machine != NULL){
	 Machine->MachineDB->ChangeValue(XMLDoc,"Acceleration",(AnsiString)anAccel);
	}
}
//---------------------------------------------------------------------------
void XLData::putAnDecel(char* data)
{///Set the analytical braking
	(strcmp(data, "") == 0) ?
		strcpy(anDecel, DEFAULT_ANDECEL):
		strcpy(anDecel, data);
	if(IsConstructed && Machine != NULL){
	 Machine->MachineDB->ChangeValue(XMLDoc,"Deceleration",(AnsiString)anDecel);
	}
}
//---------------------------------------------------------------------------
void XLData::putAccel(char* data)
{///Set the XL acceleration
	(strcmp(data, "") == 0) ?
		strcpy(accel, DEFAULT_ACCEL):
		strcpy(accel, data);
	if(IsConstructed && Machine != NULL){
	 Machine->MachineDB->ChangeValue(XMLDoc,"Acceleration",(AnsiString)accel);
	}
}
//---------------------------------------------------------------------------
void XLData::putDecel(char* data)
{///Set the XL braking
	(strcmp(data, "") == 0) ?
		strcpy(decel, DEFAULT_DECEL):
		strcpy(decel, data);
	if(IsConstructed && Machine != NULL){
	 Machine->MachineDB->ChangeValue(XMLDoc,"Deceleration",(AnsiString)decel);
	}
}
//---------------------------------------------------------------------------
void XLData::putStatus(int dlgStatus)
{///Not used at present
	status = dlgStatus;
}
//---------------------------------------------------------------------------
void XLData::putXlStatus(char stat)
{///Put the machine status
	switch (stat)
		{
			case 'R':
				xlStatus = XLRUNNING;
				break;
			case 'S':
				xlStatus = STOPPING;
				break;
			case 'A':
				xlStatus= ACCELERATING;
				break;
			case 'D':
				xlStatus= DECELERATING;
				break;
			default:
				xlStatus=0;
				break;
		}
}
//---------------------------------------------------------------------------
void XLData::PutAccelDecelDefaults()
{///Set the defaults for the acceleration/braking
	putAnAccel(DEFAULT_ANACCEL);
	putAnDecel(DEFAULT_ANDECEL);
	putAccel(DEFAULT_ACCEL);
	putDecel(DEFAULT_DECEL);
}
//===========================================================================
// Getter functions
//---------------------------------------------------------------------------
char* XLData::getSpeed()
{///Get the rpm
//	return (speed);
	AnsiString s;
	if(IsConstructed && Machine != NULL){
	 s = Machine->MachineDB->GetValue(XMLDoc,"RotorSpeed");
	}
	return s.c_str();
}
//---------------------------------------------------------------------------
int  XLData::getSpeedInt()
{///Returns the speed as an integer
//	return (atol(speed));
	AnsiString s;
	if(IsConstructed && Machine != NULL){
	 s = Machine->MachineDB->GetValue(XMLDoc,"RotorSpeed");
	}
	return s.ToInt();
}
//===========================================================================
// Handling the temperature is made difficult due to the formats used by the XLA
// The desired temperature is of the form xx, sent over as 'xx/0'. If the desired
// temperature is a single digit, then a 0 has to be prepended.
//	The actual is xx.x, but set over as 'xxx/0'. Zeros are not
// The char strings used by the old xldata sort this out by determining where
// the end of the string is.
//---------------------------------------------------------------------------
int   XLData::getTempAsInt()
{///Get the temperature as an integer
	AnsiString s;
	int i;
	if(IsConstructed && Machine != NULL){
	 s = Machine->MachineDB->GetValue(XMLDoc,"Temperature");
	 double d = s.ToDouble();
	 i = (int)d;
	}
	return i;
}
//---------------------------------------------------------------------------
char* XLData::getTemp()
{///Return the temperature
	return (getTempAsString().c_str());
}
//---------------------------------------------------------------------------
AnsiString XLData::getTempAsString()
{// returns temp as nn.n
	AnsiString s;
	if(IsConstructed && Machine != NULL){
	 s = Machine->MachineDB->GetValue(XMLDoc,"Temperature");
	}
	return s;
}
//---------------------------------------------------------------------------
double XLData::getTempAsDouble()
{/// returns the temperature as a double
	AnsiString s;
	if(IsConstructed && Machine != NULL){
	 s = Machine->MachineDB->GetValue(XMLDoc,"Temperature");
	}
	return s.ToDouble();
}
//---------------------------------------------------------------------------
char* XLData::getVac()
{///Not used, but part of the XLData structure
	return (vac);
}
//---------------------------------------------------------------------------
int   XLData::getVacInt()
{///Return it as an integer
	return (atoi(vac));
}
//---------------------------------------------------------------------------
char* XLData::getAnAccel()
{///Get the analytical acceleration
//	return (anAccel);
	AnsiString s;
	if(IsConstructed && Machine != NULL){
	 s = Machine->MachineDB->GetValue(XMLDoc,"Acceleration");
	}
	return s.c_str();
}
//---------------------------------------------------------------------------
char* XLData::getAnDecel()
{///Get the analytical braking
//	return (anDecel);
	AnsiString s;
	if(IsConstructed && Machine != NULL){
	 s = Machine->MachineDB->GetValue(XMLDoc,"Deceleration");
	}
	return s.c_str();
}
//---------------------------------------------------------------------------
char* XLData::getAccel()
{///Get the XL acceleration
//	return (accel);
	AnsiString s;
	if(IsConstructed && Machine != NULL){
	 s = Machine->MachineDB->GetValue(XMLDoc,"Acceleration");
	}
	return s.c_str();
}
//---------------------------------------------------------------------------
char* XLData::getDecel()
{///Get the XL braking
//	return (decel);
	AnsiString s;
	if(IsConstructed && Machine != NULL){
	 s = Machine->MachineDB->GetValue(XMLDoc,"Deceleration");
	}
	return s.c_str();
}
//---------------------------------------------------------------------------
int		XLData::getStatus()
{///Get the status
	return (status);
}
//---------------------------------------------------------------------------
int	XLData::getXlStatus()
{///Get the machine status
	return (xlStatus);
}
//---------------------------------------------------------------------------
char* XLData::getTime()
{///Returns the run time or elapsed time (XLActual) in seconds or 'Hold' (XLDesired)
	//return (time);
	AnsiString s;
	if(IsConstructed && Machine != NULL){
	 s = Machine->MachineDB->GetValue(XMLDoc,"Time");
	}
	return s.c_str();
}
//---------------------------------------------------------------------------
long 	XLData::getTimeHHHMM()
{///Get the run time as HHHMM
	long l;
	int i = (int)getTimeSeconds();
	AnsiString s = Util->IntToHHHMM(i);
	l = (long)s.ToInt();
	return l;
}
//---------------------------------------------------------------------------
double   XLData::getTimeSeconds()
{/// Get the run time in seconds
	double d;
	AnsiString s;
	if(IsConstructed && Machine != NULL){
	 s = Machine->MachineDB->GetValue(XMLDoc,"Time");
	}
	if(s.UpperCase() == "HOLD"){
		d = 0;
	}
	else{
		d = s.ToDouble();
	}
	return d;
}
//---------------------------------------------------------------------------
char* XLData::getXLTime()
{// returns time as HHH:MM or hold
	char theTime[99];
	char * ptheTime=&theTime[0];
	long hrs, mins;
	mins = atol(time) % 100L;
	hrs =  atol(time) / 100L;
	sprintf(theTime, "%ld:%03ld\0", hrs, mins);
	return ptheTime;
}
//---------------------------------------------------------------------------
char* XLData::getW2t()
{///Returns omega-squared t
//	return (w2t);
	AnsiString s;
	if(IsConstructed && Machine != NULL){
	 s = Machine->MachineDB->GetValue(XMLDoc,"w2t");
	}
	return s.c_str();
}
//---------------------------------------------------------------------------
char* XLData::getW2t(short mode)
{// returns w2t as m.mmEee
	char theW2t[99];
	char * ptheW2t=&theW2t[0];
	float mantissa;
	long exp, iMantissa;
	strcpy(theW2t, w2t);	// work from a local copy
	if (strcmp(theW2t, "XXXXX") == 0) strcpy(theW2t, "0");
	else {
		exp = atol(theW2t) % 100;
		iMantissa =  atol(theW2t) / 100;
		mantissa = (float) iMantissa / 100.0;
		(mode == DISPLAY_FORMAT) ?
			sprintf(theW2t, "%03.2fE%02d\0", mantissa, exp):	// display format
			sprintf(theW2t, "%7.4fE%02d\0", mantissa, exp);	// file format
	}
	return ptheW2t;
}
//---------------------------------------------------------------------------
AnsiString XLData::getAnsiW2t()
{/// returns w2t as m.mmEee
	AnsiString s;
	if(IsConstructed && Machine != NULL){
	 s = Machine->MachineDB->GetValue(XMLDoc,"w2t");
	}
	return s;
}
//---------------------------------------------------------------------------
AnsiString XLData::getAnsiTime()
{/// returns time as HH:MM or hold
	char theTime[99];
	long hrs, mins;
	AnsiString s = (AnsiString)time;
	s = s.UpperCase();
	if(s == "HOLD"){
		return "Hold";
	}
	mins = atol(time) % 100L;
	hrs =  atol(time) / 100L;
	sprintf(theTime, "%ld:%02ld\0", hrs, mins);
	s=(AnsiString)theTime;
  while(s.Length() < 6) s="0"+s;
	return s;
}
//===========================================================================
///XML read/write
//---------------------------------------------------------------------------
AnsiString XLData::GetXMLString(AnsiString type)
{///Returns the XML string for the XLData
	AnsiString s;
	AnsiString t;
	s= "<"+type+"XLData>";
	s = s+XMLUtil->XMLWrap("RotorSpeed",(int)getSpeedInt());
	s = s+XMLUtil->XMLWrap("Time",getAnsiTime());
	s = s+XMLUtil->XMLWrap("Temperature",getTempAsString());
	s = s+XMLUtil->XMLWrap("w2t",getW2t(0));
	s = s+XMLUtil->XMLWrap("Acceleration",getAccel());
	s = s+XMLUtil->XMLWrap("Deceleration",getDecel());
	s = s+XMLUtil->XMLWrap("AnalyticalAcceleration",getAnAccel());
	s = s+XMLUtil->XMLWrap("AnalyticalDeceleration",getAnDecel());
	s = s+XMLUtil->XMLWrap("Status",Machine->GetMachineStatus());
	s = s + "</"+type+"XLData>";
	return s;
}
//---------------------------------------------------------------------------
void XLData::GetXMLStrings(AnsiString type,TStrings * strngs)
{///Returns the XML strings for the XLData
	strngs->Add("<"+type+"XLData>");
	strngs->Add(XMLUtil->XMLWrap("RotorSpeed",(int)getSpeedInt()));
	strngs->Add(XMLUtil->XMLWrap("Time",getAnsiTime()));
	strngs->Add(XMLUtil->XMLWrap("Temperature",getTempAsString()));
	strngs->Add(XMLUtil->XMLWrap("w2t",getW2t(0)));
	strngs->Add(XMLUtil->XMLWrap("Acceleration",getAccel()));
	strngs->Add(XMLUtil->XMLWrap("Deceleration",getDecel()));
	strngs->Add(XMLUtil->XMLWrap("AnalyticalAcceleration",getAnAccel()));
	strngs->Add(XMLUtil->XMLWrap("AnalyticalDeceleration",getAnDecel()));
	strngs->Add(XMLUtil->XMLWrap("Status",Machine->GetMachineStatus()));
	strngs->Add("</"+type+"XLData>");
}

#pragma package(smart_init)




