#include <vcl.h>
#pragma hdrstop
//---------------------------------------------------------------------------

#include "XLDataFile.h"
//---------------------------------------------------------------------------
 __fastcall XLDataFile::XLDataFile(TComponent* Owner):DataDefinition(Owner)
{///Be sure the data pointers are NULL to begin with
  X           = NULL;                   ///Absorbance radius or wavelength
  Y           = NULL;                   ///Sample intensity or absorbance
  Z           = NULL;                   ///Standard deviation for absorbance or reference intensity
  Incidence   = NULL;                   ///Field service scan- incident detector intensity
  PMTVoltage  = NULL;                   ///Field service scan- PMT voltage
  PGAGain     = NULL;                   ///Field service scan- PGA gain setting
  SampleDelay = NULL;                   ///Field service scan- Flash lamp sample sector delay setting
  RefDelay    = NULL;                   ///Field service scan- Flash lamp reference sector delay setting
  NumberOfRecords=0;                    ///Set the number of records to zero
  ScanNumber=1;                         ///Which scan number is this... gets set by the DataFileHandler
  SetUpMemory();                        ///Create the arrays with room for interference scans
}
//---------------------------------------------------------------------------
 __fastcall XLDataFile::XLDataFile(TComponent* Owner,int cell,int chan,int det,int wave):DataDefinition(Owner,cell,chan,det,wave)
 {
  X           = NULL;                   ///Absorbance radius or wavelength
  Y           = NULL;                   ///Sample intensity or absorbance
  Z           = NULL;                   ///Standard deviation for absorbance or reference intensity
  Incidence   = NULL;                   ///Field service scan- incident detector intensity
  PMTVoltage  = NULL;                   ///Field service scan- PMT voltage
  PGAGain     = NULL;                   ///Field service scan- PGA gain setting
  SampleDelay = NULL;                   ///Field service scan- Flash lamp sample sector delay setting
  RefDelay    = NULL;                   ///Field service scan- Flash lamp reference sector delay setting
  NumberOfRecords=0;                    ///Set the number of records to zero
  SetUpMemory();                        ///Create the arrays with room for interference scans
}
//---------------------------------------------------------------------------
 __fastcall XLDataFile::~XLDataFile()
{
  if(X            != NULL)  delete[] X;                       ///Delete old array
  if(Y            != NULL)  delete[] Y;                       ///Delete old array
  if(Z            != NULL)  delete[] Z;                       ///Delete old array
  if(Incidence    != NULL)  delete[] Incidence;               ///Field service scan- incident detector intensity
  if(PMTVoltage   != NULL)  delete[] PMTVoltage;              ///Field service scan- PMT voltage
  if(PGAGain      != NULL)  delete[] PGAGain;                 ///Field service scan- PGA gain setting
  if(SampleDelay  != NULL)  delete[] SampleDelay;             ///Field service scan- Flash lamp sample sector delay setting
  if(RefDelay     != NULL)  delete[] RefDelay;                ///Field service scan- Flash lamp reference sector delay setting
}
//---------------------------------------------------------------------------
void __fastcall XLDataFile::SetHeader2(AnsiString s)
{///Set the second header line
  H2= s;
}
//---------------------------------------------------------------------------
void XLDataFile::SetUpMemory(int NumberOfPoints)
{///Create memory arrays for radial position and intensities
  if(X            != NULL)  delete[] X;                       ///Delete old array
  if(Y            != NULL)  delete[] Y;                       ///Delete old array
  if(Z            != NULL)  delete[] Z;                       ///Delete old array
  if(Incidence    != NULL)  delete[] Incidence;               ///Field service scan- incident detector intensity
  if(PMTVoltage   != NULL)  delete[] PMTVoltage;              ///Field service scan- PMT voltage
  if(PGAGain      != NULL)  delete[] PGAGain;                 ///Field service scan- PGA gain setting
  if(SampleDelay  != NULL)  delete[] SampleDelay;             ///Field service scan- Flash lamp sample sector delay setting
  if(RefDelay     != NULL)  delete[] RefDelay;                ///Field service scan- Flash lamp reference sector delay setting
  X           = new double[NumberOfPoints];                             ///Radius or wavelength
  Y           = new double[NumberOfPoints];                            ///Sample absorbance, intensity
  Z           = new double[NumberOfPoints];                             ///Standard deviation of absorbance or intensity
  Incidence   = new double[NumberOfPoints];
  PMTVoltage  = new double[NumberOfPoints];
  PGAGain     = new double[NumberOfPoints];
  SampleDelay = new double[NumberOfPoints];
  RefDelay    = new double[NumberOfPoints];
  NumberOfRecords=0;
}
//---------------------------------------------------------------------------
void __fastcall XLDataFile::Add(double r,double y1,double s1,double inc,double pmt,double gain,double sdelay,double rdelay)
{///Add another data point to the arrays
  X[NumberOfRecords]            = r;
  Y[NumberOfRecords]            = y1;
  Z[NumberOfRecords]            = s1;
  Incidence[NumberOfRecords]    = inc;
  PMTVoltage[NumberOfRecords]   = pmt;
  PGAGain[NumberOfRecords]      = gain;
  SampleDelay[NumberOfRecords]  = sdelay;
  RefDelay[NumberOfRecords]     = rdelay;
  NumberOfRecords++;
}
//---------------------------------------------------------------------------
void XLDataFile::InitializeDataArray(int NumberOfPoints)
{///Create the arrays, default to 2050 datapoints
  SetUpMemory(NumberOfPoints);
}
#pragma package(smart_init)
