
#ifndef XLDataFileH
#define XLDataFileH
#include <vcl.h>
#pragma hdrstop

#include "DataDefinition.h"
//---------------------------------------------------------------------------
class XLDataFile: public DataDefinition   //Derive as descendant of TComponent for VCL compatibility
{///Holds one scan worth of data for any of the detectors
private:
  void SetUpMemory(int NumberOfPoints=2050);                   ///Create the arrays, default to 2050 datapoints
  int NumberOfRecords;                                         ///Number of datapoints
  double * X;                                                  ///Radius
  double * Y;                                                  ///Sample intensity
  double * Z;                                                  ///Standard deviation for sample intensities
  double * Incidence;                                          ///Field service scan- incident detector intensity
  double * PMTVoltage;                                         ///Field service scan- PMT voltage
  double * PGAGain;                                            ///Field service scan- PGA gain setting
  double * SampleDelay;                                        ///Field service scan- Flash lamp sample sector delay setting
  double * RefDelay;                                           ///Field service scan- Flash lamp reference sector delay setting
  AnsiString H2;                                               ///Header 2 for XL data file
public:
  __fastcall XLDataFile(TComponent* Owner);
  __fastcall XLDataFile(TComponent* Owner,int cell,int chan,int det,int wave);
  __fastcall ~XLDataFile();
 ///Setup memory
  void InitializeDataArray(int NumberOfPoints=2050);           ///Create the arrays, default to 2050 datapoints
  int ScanNumber;                                              ///Which scan number is this... gets set by the DataFileHandler
 ///Setters/getters
  int GetNumberOfRecords(){return NumberOfRecords;}            ///Return the number of datapoints in this record
  double __fastcall GetX(int i)           {return X[i];}       ///Get the first dimension of data
  double __fastcall GetY(int i)           {return Y[i];}       ///Get the second dimension of data
  double __fastcall GetZ(int i)           {return Z[i];}       ///Get the third dimension of data
  double __fastcall GetIncidence(int i)   {return Incidence[i];}    ///Get the incident light signal
  double __fastcall GetPMTVoltage(int i)  {return PMTVoltage[i];}   ///Get the PMTVoltage for the data point
  double __fastcall GetPGAGain(int i)     {return PGAGain[i];}      ///Get the PGAGain for the data point
  double __fastcall GetSampleDelay(int i) {return SampleDelay[i];}  ///Get the delay time used for the sample
  double __fastcall GetRefDelay(int i)    {return RefDelay[i];}     ///Get the flash lamp delay time for the reference sector
  AnsiString  __fastcall GetHeader2(){return H2;}              ///Get the second header line
  void        __fastcall SetHeader2(AnsiString s);             ///Set the second header line
 ///Add another data point
  void __fastcall Add(double r,double y1,double s1,double inc=0,double pmt=0,double gain=0,double sdelay=0,double rdelay=0);           ///Add a data point
};
//---------------------------------------------------------------------------
#endif
