//---------------------------------------------------------------------------


#pragma hdrstop

#include "XLScanControl.h"
#include "Machine.h"
//#include "Experiment.h"
XLScanControl* XLScanController;
//---------------------------------------------------------------------------
__fastcall XLScanControl::XLScanControl(TComponent *Owner):TComponent(Owner)
{
  rotorHoles = 4;
  numIntScans = 0;
  intScanState = NONE;
  multiCellScan = FALSE;
  actualsObtained = FALSE;
  dataFileName [0] = '\0';
  stopXlAtEnd = FALSE;
  recalRadius = FALSE;
  XLScanner = new XLScanData(this);
  putLastScanName ("\0");
  putCurrentPosition ("\0");
  putXlaIdle (TRUE);
  putScansFinished (FALSE);
  scanStopped = FALSE;
  putScanIDType (DEFAULT_SCANIDTYPE);
  putScanStartType (NON_SEQUENTIAL);
  putTotalDataPts (0L);
  putTimer (0L);
  putRotorScans (0);
  putScansPerRotor (0);
  putScanStopped (FALSE);
  putEquDelay (0);
  putNextEquDelay (0);
  ClearControlVars (FALSE);
  XLScanner->putFilePath ("none");
  putTimeCorrectionAll (0.7);
  waitForXlActuals = TRUE;
  serviceScanRequested = 0;
  fsScanActive = FALSE;
  dabReset = FALSE;
}
//---------------------------------------------------------------------------
__fastcall XLScanControl::~XLScanControl()
{
  if(XLScanner) delete XLScanner;
};
/*********************************************************************
													S t a r t S c a n
**********************************************************************/
bool XLScanControl::StartScan (int type)
{///Checks things out before starting a scan, and then starts the scan.
  bool status;
  try
    {///Will attempt to start a scan
      Machine->XLActual->ResetRunTime();		          /// reset the int scan run timer
      ClearControlVars();                             /// clear the user stopped indicator
      scanStopped = FALSE;                            /// clear the scans finished indicator
      scansFinished = FALSE;
      scanStartType = type;                           /// save the scan start type so we can validate against it
      if (ValidateScan() != IDOK)
        return IDCANCEL;		                          ///Return IDCANCEL if there are any validation problems.
      dabReset = FALSE;
      for (short cell = 0; cell < rotorHoles; cell++)
        tcAvgs [cell] = 1.0;
      XLScanner->putScanNum ("0\0");                    /// clear the scan number before we start
      XLScanner->putScanCondition ("scan started");     /// everyding is oday; start it little buddy..
      SetDateAndTime();
      // set up and start the FIRST scan.
      putTimer (0);
      currentStep = 0;
      rotorScanNumber = 0;
      prevSpeed = -1L;
      prevNumScans = -1;
      dataCellNum = -1;				// for time estimate control
      activeStep = -1;
    if (!serviceScanRequested)
      {///Set the number of rotor holes to expect
				bool is4Hole = true;
				if(is4Hole)
			//        if(Experiment->Setup->GetNumberOfCells() == 4)
          Machine->SendXLMessage("HO4");	// send rotor holes command before settings
         else Machine->SendXLMessage("HO8");
      }
    if (SetupNextScan (NEW_ROTOR_SCAN))
      {
        scanInProgress = TRUE;
        return IDOK;
      }
    else return IDCANCEL;
  }
 catch(...)
  {
    return IDOK;
  }
}
/*********************************************************************
								C l e a r C o n t r o l V a r s
**********************************************************************/
void XLScanControl::ClearControlVars (BOOL enableRaptor)
{///Clear all controlling variables to the not scanning state.
	absRequested = FALSE;
	intScanState = NONE;
	scanRequested = FALSE;
	needToSendSettings = FALSE;
	scanInProgress = FALSE;
	putDelayedStart (FALSE);	// control indicator
	timedScan = FALSE;		// control indicator
	putScanTemp (-1);
}
/*********************************************************************
									S e n d S c a n S e t t i n g s
**********************************************************************/
void XLScanControl::SendScanSettings()
{
	switch (getScanIDType())
    {
      case ID_WAVESCAN:			// wavelength scan
        Machine->SendXLMessage("SW");
        break;
      default:           			// all others are radial
        Machine->SendXLMessage("SR");
        break;
    }
}
//---------------------------------------------------------------------------

#pragma package(smart_init)
