//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "XLSocket.h"
#include "Machine.h"                                   
#include "Utility.h"
#include "XMLUtilities.h"
#include "TextEditor.h"
#include <math.h>
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TAOSMachineSocket *AOSMachineSocket;
//---------------------------------------------------------------------------
__fastcall TAOSMachineSocket::TAOSMachineSocket(TComponent* Owner)
		: TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TAOSMachineSocket::FormCreate(TObject *Sender)
{///Create the lists that will be used to decipher commands
	ConnectedList = new TStringList();                                         ///List of connections to the program
	ServerRcvMsg  = new TStringList();                                         ///Keep a list of all server messages received
	ServerSndMsg  = new TStringList();                                         ///Keep a list of all server messages sent
	ClientRcvMsg  = new TStringList();                                         ///Keep a list of all client messages received
	ClientSndMsg  = new TStringList();                                         ///Keep a list of all client messages sent
	MachineList		= new TStringList();																				 /// List of the basic commands
	ServerAddress->Text=GetServerSocketAddress();
	ServerPort->Text=GetServerSocketPort();
	ClientAddress->Text=GetClientSocketAddress();
	ClientPort->Text=GetClientSocketPort();
	CommandDBFileName = Util->DBPath();																		///Where the ClientDataSet file is kept
	CommandDBFileName = CommandDBFileName +"\\MachineAOSCommandDB.XML";		///Complete the file name
	AOSCommandDB->FileName = CommandDBFileName;
	if(FileExists(CommandDBFileName))
		AOSCommandDB->LoadFromFile(CommandDBFileName);
		AOSCommandDB->Active = true;
		if(AOSCommandDB->Eof && AOSCommandDB->Bof){
			CreateDB();
			AOSCommandDB->SaveToFile(CommandDBFileName);
		}
	else{																																	///Need to create the db
		CreateDB();
		AOSCommandDB->SaveToFile(CommandDBFileName);
	}
	editor = new TRTFfrm(this);
	editor->Parent=EditPanel;
	editor->WindowState=wsNormal;
	editor->Align=alClient;
	TBorderIcons tempBI = editor->BorderIcons;														///Remove the border icons from the docked window
	tempBI >> biMaximize;
	tempBI >> biMinimize;
	tempBI >> biSystemMenu;
	editor->BorderIcons = tempBI;
//	editor->Panel7->Visible = false;
	MainMenu1->Merge(editor->MainMenu2);
	editor->Show();
}
//---------------------------------------------------------------------------
void __fastcall TAOSMachineSocket::FormDestroy(TObject *Sender)
{///Delete the lists of commands
	AOSServer->Active = false;                                ///Stop the server
	delete ConnectedList;                                     ///List of connections
	delete ServerRcvMsg;                                      ///Keep a list of all server messages received
	delete ServerSndMsg;                                      ///Keep a list of all server messages sent
	delete ClientRcvMsg;                                      ///Keep a list of all client messages received
	delete ClientSndMsg;                                      ///Keep a list of all client messages sent
	delete MachineList;																				/// Delete the command list
}
//=========================================================================
///Deal with the Machine Command ClientDataSet
//---------------------------------------------------------------------------
bool __fastcall TAOSMachineSocket::CommandExists(AnsiString s)
{///Returns true if there is a command string, makes the command the current record
	bool ret = true;
	if(AOSCommandDB->Filtered)
		AOSCommandDB->Filtered = false;
	if(!AOSCommandDB->Active)
		AOSCommandDB->Active = true;
	AOSCommandDB->First();																											///Start at the first record
	TLocateOptions Opts;
	Opts.Clear();
	Opts << loCaseInsensitive;																									///Search is case-insensitive
	Variant locvalues=Variant(s);
	if(!AOSCommandDB->Locate("Command",locvalues,Opts))
		ret = false;																															///Couldn't find the record
	return ret;
}
//---------------------------------------------------------------------------
bool __fastcall TAOSMachineSocket::CreateDB()
{///Create the database
	bool ret = true;
	LoadMachineList();
	if(!AddListToDB(MachineList)){
		ret = false;
	}
	return ret;
}
//---------------------------------------------------------------------------
bool __fastcall TAOSMachineSocket::AddListToDB(TStringList * thelist)
{///Loop over the list and add the entries to the command db
	bool ret = true;
	for(int i = 0;i< thelist->Count;++i)
		{///Loop over the items
			AnsiString s = thelist->Strings[i];
			if(!AddCommand(s))
				{///Failed to add it!
					ret = false;
					break;																												///User wants out
				}
		}
	return ret;
}
//---------------------------------------------------------------------------
bool __fastcall TAOSMachineSocket::AddCommand(AnsiString s)
{///Add a new command and its unique Command index
	bool ret = true;
	if(CommandExists(s))
		return ret;																															///It already esists, to just return true
 ///Want to add this command
	if(!AOSCommandDB->Active)
		AOSCommandDB->Active= true;
	try{
		AOSCommandDB->Last();
		int i = AOSCommandDB->FieldByName("CommandIndex")->AsInteger;							///Get the last index
		++i;                                                                      ///Increment it
		AOSCommandDB->Append();
		AOSCommandDB->Edit();
		AOSCommandDB->FieldByName("Command")->AsString = s;
		AOSCommandDB->FieldByName("CommandIndex")->AsInteger = i;
		AOSCommandDB->Post();
		AOSCommandDB->MergeChangeLog();
		AOSCommandDB->SaveToFile(CommandDBFileName,dfXML);
	}
	catch(...){
		ret = false;
	}
	return ret;
}
//=========================================================================
///Initialize list, load commands
//---------------------------------------------------------------------------
void __fastcall TAOSMachineSocket::LoadMachineList()
{///Load the machine list with the initial values
	MachineList->Add("Machine.GetCommandList");                                ///Returns this command list
	MachineList->Add("Machine.CheckDesiredSettings");													 ///Check to see if the desired settings differs from what's on the XLA, checks for valid values, then transmits to the XLA
	MachineList->Add("Machine.SendDesiredValues");														 ///Sets the desired values, uses CheckDesired settings to send the values to the XLA
	MachineList->Add("Machine.GetActualValues");															 ///Returns the XMLPRC struct with all of the actual settings
	MachineList->Add("Machine.ForceGetActualValues");													 ///Returns the XMLPRC struct with all of the actual settings
	MachineList->Add("Machine.GetDesiredValues");															 ///Returns the XMLPRC struct with all of the desired settings
	MachineList->Add("Machine.SetDesiredValues");															 ///Expects a structure containing the desired machine settings
	MachineList->Add("Machine.GetVacuumStatus");															 ///Returns true if the vacuum signal is available
	MachineList->Add("Machine.SetVacuum");																		 ///Set the vacuum reading from an external program- check that pumps have been started
	MachineList->Add("Machine.GetPowerStatus");																 ///Returns ON/Off?UNKNOWN depending on the hardware interfaces available
	MachineList->Add("Machine.SetPowerStatus");																 ///Sets the power status has determined by an external program
	MachineList->Add("Machine.GetMachineStatus");															 ///Returns the machine status string (Accelerating, Decelerating, Stopping, Pumping, etc.)
	MachineList->Add("Machine.SetCommPort");																	 ///Set the RS232 CommPort to be used (e.g. 'COM1'), and the timeout period (in seconds, default = 30 seconds)
	MachineList->Add("Machine.AttemptCommPortConnect");												 ///Attempt to connect to the XLA communications port
	MachineList->Add("Machine.GetCommPortStatus");														 ///Returns the status of the XLA connetion: Connected, Disconnected, Connecting
	MachineList->Add("Machine.GetTemperatureStatus");													 ///Returns structure with desired, actual, delta and time waiting since temperature was set, time to wait after temperature reached
	MachineList->Add("Machine.GetPumpingStatus");															 ///Returns a structure with the vacuum system status, vacuum reading, time since pumping started
 ///Commands relating to starting and stopping machine
	MachineList->Add("Machine.StartMachine");                              		 ///Generate a start machine command (if appropriate)
	MachineList->Add("Machine.StopMachine");                               		 ///Generate a stop machine command (if appropriate)
 ///Individual temperature setters/getters
	MachineList->Add("Machine.GetActualTemperature");                          ///Returns the XLA current temperature
	MachineList->Add("Machine.GetDesiredTemperature");                         ///Returns the XLA temperature setting
	MachineList->Add("Machine.SetDesiredTemperature");                         ///Opens the temperature setting form
	MachineList->Add("Machine.GetTemperatureTolerance");											 /// Returns the tolerance to being out of temperature before an experiment can start
	MachineList->Add("Machine.SetTemperatureTolerance");											 /// Sets the tolerance to being out of temperature before an experiment can start
 ///Individual rpm setter/getters
	MachineList->Add("Machine.GetActualRotorSpeed");                           ///Returns the current XLA rotor speed
	MachineList->Add("Machine.GetDesiredRotorSpeed");                          ///Returns the XLA rotor speed setting
	MachineList->Add("Machine.SetDesiredRotorSpeed");                          ///Opens XLA rotor speed setting form
 ///Run time setters/getters
	MachineList->Add("Machine.SetDesiredTime");																 ///Sets the desired machine run time, or Hold, which is the default (let the protocol decide how long to run)
	MachineList->Add("Machine.GetMachineTime");																 ///Returns a string with the time since starting (Hold), or time remaining if a time was set
 ///Individual Actual values
	MachineList->Add("Machine.GetOmegaSquared");                               ///Returns w2t
	MachineList->Add("Machine.GetRunTime");                                    ///Returns time since machine started spinning in seconds
	MachineList->Add("Machine.GetRunTimeString");                              ///Returns "000:00" time since machine started spinning
	MachineList->Add("Machine.GetAcceleration");															 ///Returns the acceleration value
	MachineList->Add("Machine.SetAcceleration");															 ///Set the acceleration value
	MachineList->Add("Machine.GetDeceleration");															 ///Returns the deceleration value
	MachineList->Add("Machine.SetDeceleration");															 ///Set the deceleration value
 ///Status flags
	MachineList->Add("Machine.GetStatusFlags");																 /// Returns the complete list of status flags as an XMLStruct
	MachineList->Add("Machine.IsTemperatureStable");                           ///Returns True if the temperature is within the set tolerance limit (default 0.1)
	MachineList->Add("Machine.IsHeating");																		 ///Returns true if heating
	MachineList->Add("Machine.IsCooling");																		 ///Returns true if cooling
	MachineList->Add("Machine.IsStableState");                                 ///Returns True if temperature, vacuum and the current rpm is within 100 rpm of the desired value
	MachineList->Add("Machine.IsSpeedStable");																 ///Returns true if the rotor speed has been reached and is stable
	MachineList->Add("Machine.IsRotorSpinning");                               ///Returns "True" if the rotor is spinning
	MachineList->Add("Machine.IsRotorStopping");                               ///Returns "True" if the rotor is going to 0 rpm
	MachineList->Add("Machine.IsMachineStarted");                              ///Returns "True" if the XLA has been started
	MachineList->Add("Machine.IsXLAConnected");                                ///Returns "True" if AOS & XLA communicating
	MachineList->Add("Machine.IsXLAPumping");																	 ///Returns true if the XLA pumps have been started
	MachineList->Add("Machine.XLAAbsorbanceAvailable");												 ///Returns true if the XLA absorbance system may be operated by an external program using  AOS
 ///Set/Get the update interval
	MachineList->Add("Machine.GetUpdateInterval"); 															///Returns the time interval (seconds) between updating the XLA values
	MachineList->Add("Machine.SetUpdateInterval");															///Set the period used for polling the XLA on current settings
	MachineList->Add("Machine.UpdateParameters"); 															///Force the XLA to transmit its current desired and actual settings
}
//---------------------------------------------------------------------------
void __fastcall TAOSMachineSocket::GetConnections(TCustomWinSocket *Socket)
{///Will return the list of connections to the server
	int iIndex = GetSocketIndex(Socket);                                        ///Find out who sent the message
	if(iIndex == -1) return;                                                    ///If there isn't a socket, bail!
	for(int i=0;i<ConnectedList->Count;++i)
		{///Send out the list of systems one by one
			AnsiString s ="<AOS, "+ConnectedList->Strings[i]+">";
			SendMessage(s,iIndex);
		}
}
//---------------------------------------------------------------------------
void __fastcall TAOSMachineSocket::ShowXML(TXMLDocument * d,AnsiString sub)
{/// Show the current XMLDocument1 in TreeView1, display the calling subroutine
	if(Visible){																																/// Only try if visible
		SubTxt->Caption = sub;																										/// Show the calling function
		XMLUtil->XML2Tree(TreeView1,d);																						/// Show what the XMLDoc looks like
	}
}
//=========================================================================
///Machine commands
//---------------------------------------------------------------------------
void __fastcall TAOSMachineSocket::MachineCommandHandler(TCustomWinSocket *Socket,AnsiString t)
{///Deal with the command contained in this message
	TStringList * strngs = new TStringList;																				/// Use a strings object for working with the XMLUtilities
	TXMLDocument * xmldoc = new TXMLDocument(this);
	XMLUtil->XMLRPCStringToDOM(t,xmldoc);
	ShowXML(xmldoc,"MachineCommandHandler");
	try{
		AnsiString cmd;
		AnsiString value;
		if(!XMLUtil->GetMethodName(xmldoc,&cmd)){																		/// Returns the command
			SendFailure(Socket,"XMLUtil->GetMethodName");
			throw Exception("");
		}
		XMLUtil->StartAOSXMLRPCMethodResponse(cmd,strngs);													/// Start the AOS XMLRPC Method Response
		if(cmd == "Machine.GetCommandList"){ 																				/// Client wants the list of commands available for this system
			GetMachineCommandList(strngs);                                        		/// Return the list of commands
		}
	 /// Getters
		if(cmd == "Machine.GetActualTemperature"){ 																	///Client wants the current actual temperature
			value = XMLUtil->XMLRPCStructMember("ActualTemperature",Machine->GetActualTemperatureAsDouble());
			strngs->Add(value);
		}
		if(cmd == "Machine.GetDesiredTemperature"){																	/// Client wants the desired temperature
			value = XMLUtil->XMLRPCStructMember("DesiredTemperature",Machine->GetDesiredTemperatureAsDouble());
			strngs->Add(value);
		}
		if(cmd == "Machine.GetTemperatureTolerance"){																/// Client wants the temperature tolerance
			value = XMLUtil->XMLRPCStructMember("TemperatureTolerance",Machine->TemperatureSystem->TemperatureTolerance);
			strngs->Add(value);
		}
		if(cmd == "Machine.GetActualRotorSpeed"){																		///Client wants the current actual rotor speed
			value = XMLUtil->XMLRPCStructMember("ActualRotorSpeed",Machine->GetActualRotorSpeed());
			strngs->Add(value);
		}
		if(cmd == "Machine.GetDesiredRotorSpeed"){																	///Client wants the current actual rotor speed
			value = XMLUtil->XMLRPCStructMember("DesiredRotorSpeed",Machine->GetDesiredRotorSpeed());
			strngs->Add(value);
		}
		if(cmd == "Machine.GetVacuumStatus"){																				///Client wants the current vacuum
			value = XMLUtil->XMLRPCStructMember("VacuumStatus",Machine->VacuumStatus);
			strngs->Add(value);
		}
		if(cmd == "Machine.GetMachineStatus"){																			/// Client wants the current machine status
			value = XMLUtil->XMLRPCStructMember("MachineStatus",Machine->GetMachineStatus());
			strngs->Add(value);
		}
		if(cmd == "Machine.GetOmegaSquared"){																				///Client wants the value of Omega Squared
			value = XMLUtil->XMLRPCStructMember("OmegaSquared",Machine->GetW2tAsDouble());
			strngs->Add(value);
		}
		if(cmd == "Machine.GetRunTimeString"){																			///Client wants the Run Time
			value = XMLUtil->XMLRPCStructMember("RunTimeString",Machine->GetActualTimeAsString());
			strngs->Add(value);
		}
		if(cmd == "Machine.GetRunTime"){																						///Client wants the Run Time
			AnsiString ss;
			ss = ss.FormatFloat("0000000",(Util->GetTimeSeconds()-Machine->StartSpinningTime)),
			value = XMLUtil->XMLRPCStructMember("RunTime",ss);
			strngs->Add(value);
		}
		if(cmd == "Machine.IsHeating"){																							///Client wants to know if the temperature is within the spec
			bool b;
			if(Machine->TemperatureSystem->TemperatureDifference > 0) b = true;
			else b = false;
			value = XMLUtil->XMLRPCStructMember("IsHeating",b);
			strngs->Add(value);
		}
		if(cmd == "Machine.IsCooling"){																							///Client wants to know if the temperature is within the spec
			bool b;
			if(Machine->TemperatureSystem->TemperatureDifference > 0) b = false;
			else b = true;
			value = XMLUtil->XMLRPCStructMember("IsCooling",b);
			strngs->Add(value);
		}
		if(cmd == "Machine.IsStableState"){																					///Client wants to know if the system is stable within the spec
			bool b =Machine->CheckForStableState(Machine->GetDesiredRotorSpeed());
			value = XMLUtil->XMLRPCStructMember("IsStableState",b);
			strngs->Add(value);
		}
		if(cmd == "Machine.IsRotorSpinning"){																							///Client wants to know if the temperature is within the spec
			bool b = Machine->bRotorSpinning;
			value = XMLUtil->XMLRPCStructMember("IsRotorSpinning",b);
			strngs->Add(value);
		}
		if(cmd == "Machine.IsRotorStopping"){																				///Client wants to know if the temperature is within the spec
			bool b = Machine->bStopping;
			value = XMLUtil->XMLRPCStructMember("IsRotorStopping",b);
			strngs->Add(value);
		}
		if(cmd == "Machine.IsMachineStarted"){																			///Client wants to know if the temperature is within the spec
			bool b = Machine->bMachineStarted;
			value = XMLUtil->XMLRPCStructMember("IsMachineStarted",b);
			strngs->Add(value);
		}
		if(cmd == "Machine.IsXLAPumping"){																					///Client wants to know if the temperature is within the spec
			bool b;
			if(Machine->VacuumSystem->bPumpDown
				|| Machine->VacuumSystem->bVacuumOK) b = true;
			else b = false;
			value = XMLUtil->XMLRPCStructMember("IsXLAPumping",b);
			strngs->Add(value);
		}
		if(cmd == "Machine.XLAAbsorbanceAvailable"){																///Client wants to know if the Beckman XLA absorbance optics are available
			bool b = Machine->XLAAbsorbanceAvailable;
			value = XMLUtil->XMLRPCStructMember("XLAAbsorbanceAvailable",b);
			strngs->Add(value);
		}
		if(cmd == "Machine.IsXLAConnected"){																				///Client wants to know if XLA/AOS connection established
			bool b = Machine->CommunicationsEstablished();
			value = XMLUtil->XMLRPCStructMember("IsXLAConnected",b);
			strngs->Add(value);
		}
	 /// Setters
		if(cmd == "Machine.SetDesiredTemperature"){																	///Client wants to set the desired temperature
			IXMLNode * node = xmldoc->Node;
			if(!XMLUtil->GetXMLRPCValueNode(node))
				SendFailure(Socket,cmd);
			double val = XMLUtil->GetXMLNodeDouble(node,"double");										///Get the value
			if(SetTemperature(val))																										/// Return success
				SendSuccess(Socket,cmd+": "+(AnsiString)val);														/// Let them know it succeeded
			else
				SendFailure(Socket,cmd);
		}
		if(cmd == "Machine.SetTemperatureTolerance"){																///Client wants to set the desired temperature
			IXMLNode * node = xmldoc->Node;
			if(!XMLUtil->GetXMLRPCValueNode(node))
				SendFailure(Socket,cmd);
			double val = XMLUtil->GetXMLNodeDouble(node,"double");										///Get the value
			if(SetTemperatureTolerance(val))																										/// Return success
				SendSuccess(Socket,cmd+": "+(AnsiString)val);														/// Let them know it succeeded
			else
				SendFailure(Socket,cmd);
		}
		if(cmd == "Machine.SetDesiredRotorSpeed"){																	///Client wants to set the rotor speed
			IXMLNode * node = xmldoc->Node;
			if(!XMLUtil->GetXMLRPCValueNode(node)){
				SendFailure(Socket,cmd);
			}
			double val = XMLUtil->GetXMLNodeDouble(node,"double");										///Get the value
			if(SetRotorSpeed(val)){                                              			///Returns true/false depending on whether rpm button can be clicked
				SendSuccess(Socket, cmd+": "+(AnsiString)val);
			}
			else{
				SendFailure(Socket,cmd);                                                /// Send a failure packet if it doesn't work
			}
		}
	}
	catch(...){
		delete xmldoc;
		delete strngs;																															/// Use a strings object for working with the XMLUtilities
	}
	XMLUtil->EndAOSXMLRPCMethodResponse(strngs);																	/// Wrap up the Method Response
	SendMessage(strngs,Socket);
	delete xmldoc;
	delete strngs;																																/// Use a strings object for working with the XMLUtilities
return;                                                                 				///Done... bail
}
//---------------------------------------------------------------------------
bool __fastcall TAOSMachineSocket::SetTemperatureTolerance(AnsiString cmd)
{/// Will set the temperature range allowed for an experiment to start
	bool ret = true;
	double temp = cmd.ToDouble();
	if(temp < 0 || temp > 2.0)
		return false;																															///Temperature is out of range
	Machine->TemperatureSystem->TemperatureTolerance = temp;
	return ret;
}
//---------------------------------------------------------------------------
bool __fastcall TAOSMachineSocket::SetTemperature(AnsiString cmd)
{///If the Temp button can be clicked, set the desired temperature to the value in cmd. Return true if successful
	bool ret = true;
	int temp = cmd.ToInt();
	if(temp < 0 || temp > 40)
		return false;																															///Temperature is out of range
	if(Machine->MachineStatus->TBtn->Enabled){																	///Client wants to open the rotor speed settings box
		AnsiString OldDesired=Machine->GetDesiredTemperatureAsString();
		Machine->SetTemperature(cmd.ToInt());
//		if(OldDesired != Machine->DesiredTemperature){													///Log a temperature change
//			Experiment->RunLog->AddLog(24);
//		}
		Machine->MachineStatus->UpdateDisplay();
		Machine->CheckDesiredSettings();
	}                                                                         /// Set the value and let the software know
	else ret = false;                               														/// The RpmBtn could not be clicked
	return ret;
}
//---------------------------------------------------------------------------
bool __fastcall TAOSMachineSocket::SetRotorSpeed(AnsiString cmd)
{///If the Rpm button can be clicked, return "True" as the value, else return "False"
	bool ret = true;
	int temp = cmd.ToInt();
	int MaxSpeed = 60000;                                  ///Make sure the rpm cannot be set higher than the rotor type allows
	if(temp < 0 || temp > MaxSpeed)
		return false;
	if(Machine->MachineStatus->RpmBtn->Enabled){																///Client wants to open the rotor speed settings box
		AnsiString OldDesired= Machine->GetDesiredRotorSpeed();
		Machine->RequestedRotorSpeed= cmd;
//		if(OldDesired != Machine->DesiredRotorSpeed)														///Log a rotor speed change
//			Experiment->RunLog->AddLog(22);
		Machine->MachineStatus->UpdateDisplay();
		Machine->CheckDesiredSettings();
	}
	else ret = false;                               														/// The RpmBtn could not be clicked
	return ret;
}
//---------------------------------------------------------------------------
void __fastcall TAOSMachineSocket::GetMachineCommandList(TStringList * strngs)
{///Will return the command strings for the Machine system
	try{/// Try to create the message response
		if(!AOSCommandDB->Active){																								/// Prepare to loop over the command db
			AOSCommandDB->Active = true;
		}
		AOSCommandDB->First();
		while(!AOSCommandDB->Eof){																								/// Loop over the db and append the command strings
			AnsiString s = AOSCommandDB->FieldByName("Command")->AsString;
			strngs->Add(XMLUtil->XMLRPCStructMember("Command",s));
			AOSCommandDB->Next();
		}
	}
	catch(...){/// Couldn't create the message
		strngs->Clear();
		XMLUtil->CreateXMLRPCResponseHeader(strngs);
		AnsiString cmd = "Error processing Machine.GetCommandList";
		XMLUtil->XMLRPCFaultResponse(100,cmd,strngs);
	}
}
//=========================================================================
///System message handlers
//---------------------------------------------------------------------------
bool __fastcall TAOSMachineSocket::IsValidMessage(AnsiString msg)
{/// Make sure this is a valid XMLRPC message
	if(!XMLUtil->IsXMLRPC(msg)
		|| XMLUtil->FindXMLStart(msg) == 0)
		return false;
	return true;
}
//===========================================================================
//Message handler- Dispatches to the correct message handler
//---------------------------------------------------------------------------
void __fastcall TAOSMachineSocket::MessageHandler(AnsiString msg,TCustomWinSocket *Socket)
{///Determine which command handler will deal with this message and get it done
	if(!XMLUtil->XMLRPCStringToDOM(msg,XMLDocument1)){
		SendFailure(Socket,"MessageHandler, XMLRPCStringToDOM"+msg);
		ShowXML(XMLDocument1,"MessageHandler:StringToDOM fail");
		return;																																		/// No use trying
	}
	ShowXML(XMLDocument1,"MessageHandler:StringToDOM success");
	AnsiString cmd;
	if(!XMLUtil->GetMethodName(XMLDocument1,&cmd)){
		SendFailure(Socket,"MessageHandler, GetMethodName");											/// Returns the command
		ShowXML(XMLDocument1,"MessageHandler:GetMethodName failure");
	}
	if(cmd == " " || cmd == NULL){
		return;																																		/// No command string found
	}
	if(!CommandExists(cmd)){
			TStringList * strngs = new TStringList;
			ShowXML(XMLDocument1,"Command: "+cmd+" does not exist");
			XMLUtil->CreateXMLRPCResponseHeader(strngs);
			cmd = cmd+ " not recognized";
			XMLUtil->XMLRPCFaultResponse(100,cmd,strngs);
			SendMessage(strngs,Socket);
			delete strngs;
			return;																																	/// Check that the command is recognized
	}
	MachineCommandHandler(Socket,msg);																					/// The handler creates a temporary XMLDocument to contains the data
}
//=========================================================================
///Server Read/Write commands
//---------------------------------------------------------------------------
void __fastcall TAOSMachineSocket::SendValue(TCustomWinSocket *Socket,AnsiString value)
{///Packages and sends the value to the socket
	TStringList * strngs = new TStringList;																			/// Use a strings object for working with the XMLUtilities
	try{/// Try to create the message response
		XMLUtil->CreateXMLRPCResponseHeader(strngs);														/// Create the response header
		strngs->Add(XMLUtil->XMLRPCStartMethodResponse());											/// Add the <methodResponse>"+XMLRPCStartParams() strings
		strngs->Add(XMLUtil->XMLRPCWrapParam(value));														/// Provide the <value> value </value>, the value must be wrapped in <type> </type> statements
		strngs->Add(XMLUtil->XMLRPCEndMethodResponse());
	}
	catch(...){/// Couldn't create the message
		delete strngs;
		SendFailure(Socket,value);
		return;
	}
	SendMessage(strngs,Socket);
	delete strngs;
}
//---------------------------------------------------------------------------
void __fastcall TAOSMachineSocket::SendSuccess(TCustomWinSocket *Socket,AnsiString val)
{/// Return success code to the caller
	TStringList * strngs = new TStringList;
	AnsiString cmd = "Success: " +val;
	XMLUtil->CreateXMLRPCResponseHeader(strngs);														/// Create the response header
	strngs->Add(XMLUtil->XMLRPCStartMethodResponse());											/// Add the <methodResponse>"+XMLRPCStartParams() strings
	strngs->Add(XMLUtil->XMLRPCWrapParam(cmd));															/// Provide the <value> value </value>, the value must be wrapped in <type> </type> statements
	strngs->Add(XMLUtil->XMLRPCEndMethodResponse());
	SendMessage(strngs,Socket);
	delete strngs;
}
//---------------------------------------------------------------------------
void __fastcall TAOSMachineSocket::SendFailure(TCustomWinSocket *Socket,AnsiString val)
{/// Return success code to the caller
	TStringList * strngs = new TStringList;
	XMLUtil->CreateXMLRPCResponseHeader(strngs);
	AnsiString cmd = "Set error:" +val;
	XMLUtil->XMLRPCFaultResponse(100,cmd,strngs);
	SendMessage(strngs,Socket);
	delete strngs;
}
//---------------------------------------------------------------------------
int __fastcall TAOSMachineSocket::GetSocketIndex(TCustomWinSocket * Socket)
{///Returns the index of the socket, returns -1 if socket is not already on the list
	int iIndex=-1;
	iIndex = ConnectedList->IndexOfObject(Socket);
	return iIndex;
}
//---------------------------------------------------------------------------
void __fastcall TAOSMachineSocket::AOSServerClientRead(TObject *Sender,
			TCustomWinSocket *Socket)
{///Command has come in from an external program
	int iIndex=GetSocketIndex(Socket);                                          ///See if this socket is recognized
	if(iIndex == -1) return;                                                    ///Is not a recognized socket
	AnsiString TextIn= Socket->ReceiveText();                                   ///Message string
// 	if(TextIn.SubString(1,3) == "GET") return;
	if(Visible) ServerRcvMsg->Add(TextIn);                                      ///Display the message if the user has the Sockets box open
	MessageHandler(TextIn,Socket);
}
//---------------------------------------------------------------------------
bool __fastcall TAOSMachineSocket::HasConnections()
{///Returns true if the socket is active and has connections
  bool ret = true;
  if(!AOSServer->Active) ret=false;                                           ///Done sent a message if the server is not active
  if(ConnectedList->Count == 0) ret=false;                                    ///Don't send a message if there is no one to receive it
  return ret;
}
//---------------------------------------------------------------------------
void __fastcall TAOSMachineSocket::BroadcastMessage(TStrings * strngs)
{///Send the message to all sockets
	AnsiString s = XMLUtil->StringList2String(strngs);
	BroadcastMessage(s);
}
//---------------------------------------------------------------------------
void __fastcall TAOSMachineSocket::BroadcastMessage(AnsiString aMessage)
{///Send the message to all sockets
	if(!HasConnections()) return;                                               ///Don't bother if no one's home
	for(int i=0;i<AOSServer->Socket->ActiveConnections;i++)
		AOSServer->Socket->Connections[i]->SendText(aMessage);
}
//---------------------------------------------------------------------------
void __fastcall TAOSMachineSocket::SendMessage(TStrings * strngs,TCustomWinSocket *Socket)
{/// Pack the strngs into one string, get the index, use SendMessage(s,int i)
	int iIndex = GetSocketIndex(Socket);                                        ///Find out who sent the message
	if(iIndex == -1) return; 																									 	///If there isn't a socket, bail!
	AnsiString s = XMLUtil->StringList2String(strngs);
	SendMessage(s,iIndex);
}
//---------------------------------------------------------------------------
void __fastcall TAOSMachineSocket::SendMessage(AnsiString aMessage, int iTo)
{///Send the message out the Server socket
  if(!HasConnections()) return;                                               ///Don't bother if no one's home
  TCustomWinSocket * out=(TCustomWinSocket *)ConnectedList->Objects[iTo];
  out->SendText(aMessage);
	if(Visible){
		ServerSndMsg->Add(aMessage);
	}
}
//---------------------------------------------------------------------------
void __fastcall TAOSMachineSocket::SendClientMessage(AnsiString msg)
{///Send the message out the Client socket
  AOSClient->Socket->SendText(msg);
  ClientSndMsg->Add(msg);
}
//=========================================================================
///Button handlers
//---------------------------------------------------------------------------
void __fastcall TAOSMachineSocket::FormClose(TObject *Sender,
      TCloseAction &Action)
{
  Action = caHide;
}
//---------------------------------------------------------------------------
void __fastcall TAOSMachineSocket::ClearBtnClick(TObject *Sender)
{///Clear memo fields
	editor->TREOle->CloseActiveObjects(false);
	ClearAll();
}
//---------------------------------------------------------------------------
void __fastcall TAOSMachineSocket::ConnectButtonClick(TObject *Sender)
{///Connect the
  AOSClient->Address = ClientAddress->Text;                                   ///Set the address to the value in the Address text box (default 127.0.0.1)
  AOSClient->Port    = ClientPort->Text.ToIntDef(10000);                      ///Set the port to the value in the Port text box (default 10000)
  AOSClient->Active  = true;                                                  ///Open up the port, sends a connect request
  ConnectButton->Enabled = false;                                             ///Set the button states correctly
  DisconnectButton->Enabled = true;
	ClientAddress->Enabled=false;
  ClientPort->Enabled=false;
}
//---------------------------------------------------------------------------
void __fastcall TAOSMachineSocket::DisconnectButtonClick(TObject *Sender)
{
  AOSClient->Active = false;
  ConnectButton->Enabled = true;
  DisconnectButton->Enabled = false;
  ClientAddress->Enabled=true;
  ClientPort->Enabled=true;
}
//---------------------------------------------------------------------------
void __fastcall TAOSMachineSocket::Connectionstoserver1Click(
      TObject *Sender)
{
	if(Visible)
		editor->RichEdit1->Lines->Assign(ConnectedList);
}
//---------------------------------------------------------------------------

void __fastcall TAOSMachineSocket::Messagesreceivedbyserver1Click(
			TObject *Sender)
{
	if(Visible)
		editor->RichEdit1->Lines->Assign(ServerRcvMsg);
}
//---------------------------------------------------------------------------

void __fastcall TAOSMachineSocket::Messagessentbyserver1Click(
			TObject *Sender)
{
	if(Visible)
		editor->RichEdit1->Lines->Assign(ServerSndMsg);
	}
//---------------------------------------------------------------------------
void __fastcall TAOSMachineSocket::AOSClientRead(TObject *Sender,
			TCustomWinSocket *Socket)
{
	AnsiString msg = Socket->ReceiveText();
	ClientRcvMsg->Add(msg);
}
//---------------------------------------------------------------------------
void __fastcall TAOSMachineSocket::AOSClientError(TObject *Sender,
      TCustomWinSocket *Socket, TErrorEvent ErrorEvent, int &ErrorCode)
{
 ErrorCode=0;
}
//---------------------------------------------------------------------------
void __fastcall TAOSMachineSocket::Messagessentbyclient1Click(
      TObject *Sender)
{
	if(Visible)
		editor->RichEdit1->Lines->Assign(ClientSndMsg);
}
//---------------------------------------------------------------------------
void __fastcall TAOSMachineSocket::ClientMessagesreceived1Click(
			TObject *Sender)
{
	if(Visible)
		editor->RichEdit1->Lines->Assign(ClientRcvMsg);

}
//---------------------------------------------------------------------------
void __fastcall TAOSMachineSocket::Saveaddress1Click(TObject *Sender)
{
	SetServerSocketAddress(ServerAddress->Text);
}
//---------------------------------------------------------------------------
void __fastcall TAOSMachineSocket::Saveport1Click(TObject *Sender)
{
	SetServerSocketPort(ServerPort->Text);
}
//---------------------------------------------------------------------------
void __fastcall TAOSMachineSocket::Savesettings1Click(TObject *Sender)
{
	Saveaddress1Click(this);
	Saveport1Click(this);
}
//---------------------------------------------------------------------------
void __fastcall TAOSMachineSocket::Saveclientaddress1Click(TObject *Sender)
{
	SetClientSocketAddress(ClientAddress->Text);
}
//---------------------------------------------------------------------------

void __fastcall TAOSMachineSocket::Saveclientport1Click(TObject *Sender)
{
	SetClientSocketPort(ClientPort->Text);
}
//---------------------------------------------------------------------------
void __fastcall TAOSMachineSocket::Saveclientsettings1Click(TObject *Sender)
{
	Saveclientaddress1Click(this);
  Saveclientport1Click(this);
}
//---------------------------------------------------------------------------

void __fastcall TAOSMachineSocket::ViewCBKeyDown(TObject *Sender, WORD &Key,
      TShiftState Shift)
{///Don't allow editing
	return;	
}
//---------------------------------------------------------------------------

void __fastcall TAOSMachineSocket::ViewCBKeyPress(TObject *Sender, char &Key)
{///Don't allow editing
	return;
}
//---------------------------------------------------------------------------
void __fastcall TAOSMachineSocket::ViewCBSelect(TObject *Sender)
{///User has selected what items to view
	if(!Visible) return;
	AnsiString s = ViewCB->Text;
	if(s == "Connections to server")
		{///Want to see the server connections
			editor->RichEdit1->Lines->Assign(ConnectedList);
			return;
		}
	if(s == "Server- Messages received")
		{
			editor->RichEdit1->Lines->Assign(ServerRcvMsg);
			return;
		}
	if(s == "Server- Messages sent")
		{
			editor->RichEdit1->Lines->Assign(ServerSndMsg);
			return;
		}
	if(s == "Client- Messages sent")
		{
			editor->RichEdit1->Lines->Assign(ClientSndMsg);
			return;
		}
	if(s == "Client- Messages received")
		{
			editor->RichEdit1->Lines->Assign(ClientRcvMsg);
			return;
		}
	if(s == "Clear all messages")
		{
			ClearAll();
			return;
		}
}
//===========================================================================
// Button and menu clicks
//---------------------------------------------------------------------------
void __fastcall TAOSMachineSocket::ClearAll()
{///Clear all of the message buffers
	editor->RichEdit1->Lines->Clear();
	ServerRcvMsg->Clear();
	ServerSndMsg->Clear();
	ClientSndMsg->Clear();
	ClientRcvMsg->Clear();
}
//---------------------------------------------------------------------------
void __fastcall TAOSMachineSocket::Showtextview1Click(TObject *Sender)
{///Show/hide the LogMemo
	Showtextview1->Checked = !Showtextview1->Checked;
	editor->RichEdit1->Visible = Showtextview1->Checked;
}
//---------------------------------------------------------------------------
void __fastcall TAOSMachineSocket::ShowXMLview1Click(TObject *Sender)
{///Show/hide the XML view
	ShowXMLview1->Checked = !ShowXMLview1->Checked;
	TreeView1->Visible = ShowXMLview1->Checked;
}
//=========================================================================
///Server Start/Stop commands
//---------------------------------------------------------------------------
void __fastcall TAOSMachineSocket::StartButtonClick(TObject *Sender)
{///Start the server- disable buttons, etc. that could cause a crash
	AOSServer->Port = ServerPort->Text.ToIntDef(10000);                         ///Set the port to the user value or else to the default value 10000
	Caption = "AOS Server- "+AOSServer->Port;                                   ///Show the port value
	AOSServer->Active = true;                                                   ///Start it running
	StopButton->Enabled = true;                                                 ///Allow the server to be stopped
	StartButton->Enabled = false;                                               ///Cannot perform start on an already started port
	ServerPort->Enabled = false;                                                ///Must not change the port number when it is active
	ServerAddress->Enabled = false;                                             ///Must not change the port number when it is active
}
//---------------------------------------------------------------------------
void __fastcall TAOSMachineSocket::StopButtonClick(TObject *Sender)
{///Stop the server- set buttons, etc. to allow operations
	AOSServer->Active = false;                                                  ///Stop the server
	StartButton->Enabled = true;                                                ///Allow it to be re-started
	StopButton->Enabled = false;                                                ///No reason to stop a stopped port
	ServerPort->Enabled = true;                                                 ///Allow the port number to be
	ServerAddress->Enabled = true;                                              ///Must not change the port number when it is active
}
//---------------------------------------------------------------------------
void __fastcall TAOSMachineSocket::StartServer()
{///Calls StartButtonClick
	StartButtonClick(this);
}
//---------------------------------------------------------------------------
void __fastcall TAOSMachineSocket::StopServer()
{///Calls StopButtonClick
	StopButtonClick(this);
}
//=========================================================================
///Client Connect/Disconnect commands
//---------------------------------------------------------------------------
void __fastcall TAOSMachineSocket::AOSServerClientConnect(TObject *Sender,
			TCustomWinSocket *Socket)
{///An outside program is trying to connect
	if(Visible)
		editor->RichEdit1->Lines->Add(Socket->RemoteAddress + ": has connected.");///Add note to the list box
	ConnectedList->AddObject(Socket->RemoteAddress, Socket);                    ///Add this connection to the ConnectedList
}
//---------------------------------------------------------------------------
void __fastcall TAOSMachineSocket::AOSServerClientDisconnect(TObject *Sender,
			TCustomWinSocket *Socket)
{
	int iIndex = GetSocketIndex(Socket);                                        ///Get the index of the socket
	if(iIndex == -1) return;                                                    ///Check that a valid socket was found
	if(Visible) editor->RichEdit1->Lines->Add(Socket->RemoteAddress + " has disconnected.");
//  SendMessage(s,iIndex);
	ConnectedList->Delete(iIndex);
}
//===========================================================================
///Registry functions
//---------------------------------------------------------------------------
TRegistry * TAOSMachineSocket::GetRegistry()
{///Utility function to create the registry key values and open the key. Deletes registry and sets it NULL if unsuccessful
	AnsiString KeyName;
	TRegistry *Registry = new TRegistry(KEY_READ);
	Registry->RootKey=HKEY_LOCAL_MACHINE;           /// Keep AUC-AOS information here so globally available
	KeyName="SOFTWARE\\CAMIS\\AUC\\XL\\Config";     /// Registry key where configuration parameters kept.
	Registry->Access=KEY_ALL_ACCESS;                /// Need to make sure that any user logged into computer (not software) has access
	try
		{
			Registry->OpenKey(KeyName,true);            /// False because we do not want to create it if it doesn't exist
		}
	catch(...)
		{
			if(Registry) delete Registry;
			Registry = NULL;
		}
	return Registry;
}
//===========================================================================
///Server socket
//---------------------------------------------------------------------------
AnsiString TAOSMachineSocket::GetServerSocketAddress()
{///Returns the ServerSocket address
	AnsiString S="127.0.0.1";
	TRegistry *Registry = GetRegistry();
	if(Registry)
		{
			if(Registry->ValueExists("ServerSocketAddress"))
				S = Registry->ReadString("ServerSocketAddress");
			else SetServerSocketAddress(S);
			Registry->CloseKey();
			delete Registry;
		}
	return S;
}
//---------------------------------------------------------------------------
bool TAOSMachineSocket::SetServerSocketAddress(AnsiString msg)
{///Save the Server socket address in the registry
	bool ret=true;
	TRegistry *Registry = GetRegistry();
	if(Registry)
		{///Try to register
			Registry->WriteString("ServerSocketAddress",msg);
			Registry->CloseKey();
			delete Registry;
		}
	else ret=false;
	return ret;
}
//---------------------------------------------------------------------------
AnsiString TAOSMachineSocket::GetServerSocketPort()
{///Returns the ServerSocket port
	AnsiString S="10000";
	TRegistry *Registry = GetRegistry();
	if(Registry)
		{
			if(Registry->ValueExists("ServerSocketPort"))
				S = Registry->ReadString("ServerSocketPort");
			else SetServerSocketPort(S);
			Registry->CloseKey();
			delete Registry;
		}
	return S;
}
//---------------------------------------------------------------------------
bool TAOSMachineSocket::SetServerSocketPort(AnsiString msg)
{///Save the server socket port in the registry
	bool ret=true;
	TRegistry *Registry = GetRegistry();
	if(Registry)
		{///Try to register
			Registry->WriteString("ServerSocketPort",msg);
			Registry->CloseKey();
			delete Registry;
		}
	else ret=false;
	return ret;
}
//===========================================================================
///Client socket
//---------------------------------------------------------------------------
AnsiString TAOSMachineSocket::GetClientSocketAddress()
{///Returns the ServerSocket address
	AnsiString S="127.0.0.1";
	TRegistry *Registry = GetRegistry();
	if(Registry)
		{
			if(Registry->ValueExists("ClientSocketAddress"))
				S = Registry->ReadString("ClientSocketAddress");
			else SetClientSocketAddress(S);
			Registry->CloseKey();
			delete Registry;
		}
	return S;
}
//---------------------------------------------------------------------------
bool TAOSMachineSocket::SetClientSocketAddress(AnsiString msg)
{///Save the Server socket address in the registry
	bool ret=true;
	TRegistry *Registry = GetRegistry();
	if(Registry)
		{///Try to register
			Registry->WriteString("ClientSocketAddress",msg);
			Registry->CloseKey();
			delete Registry;
		}
	else ret = false;
	return ret;
}
//---------------------------------------------------------------------------
AnsiString TAOSMachineSocket::GetClientSocketPort()
{///Returns the ServerSocket port
	AnsiString S="10001";
	TRegistry *Registry = GetRegistry();
	if(Registry)
		{
			if(Registry->ValueExists("ClientSocketPort"))
				S = Registry->ReadString("ClientSocketPort");
			else SetClientSocketPort(S);
			Registry->CloseKey();
			delete Registry;
		}
	return S;
}
//---------------------------------------------------------------------------
bool TAOSMachineSocket::SetClientSocketPort(AnsiString msg)
{///Save the server socket port in the registry
	bool ret=true;
	TRegistry *Registry = GetRegistry();
	if(Registry)
		{///Try to register
			Registry->WriteString("ClientSocketPort",msg);
			Registry->CloseKey();
			delete Registry;
		}
	else ret = false;
	return ret;
}
//---------------------------------------------------------------------------
void __fastcall TAOSMachineSocket::Exit1Click(TObject *Sender)
{
	Hide();
}
//---------------------------------------------------------------------------
void __fastcall TAOSMachineSocket::BitBtn1Click(TObject *Sender)
{///... Send a test message to all connected clients
	TStringList * slist = new TStringList();
	XMLUtil->CreateXMLRPCResponseHeader(slist);
	slist->Add("Welcome to the AOS XLA Socket "+Util->GetCurrentVersion());
	BroadcastMessage(slist);
	delete slist;
}
//---------------------------------------------------------------------------

void __fastcall TAOSMachineSocket::AOSServerAccept(TObject *Sender,
			TCustomWinSocket *Socket)
{
	TStringList * slist = new TStringList();
	XMLUtil->CreateXMLRPCResponseHeader(slist);
	slist->Add("Welcome to the AOS XLA Socket "+Util->GetCurrentVersion());
	XMLUtil->SetContentLength(slist);
	SendMessage(slist,Socket);
	delete slist;
}
//---------------------------------------------------------------------------

void __fastcall TAOSMachineSocket::FormShow(TObject *Sender)
{///Set the buttons correctly
	StartButton->Enabled = !AOSServer->Active;
	StopButton->Enabled = !StartButton->Enabled;

}
//---------------------------------------------------------------------------
void __fastcall TAOSMachineSocket::Viewavailablecommands1Click(TObject *Sender)
{/// View the AOSCommandDB in the tree
	AnsiString fn = Util->DBPath();																				/// Where the ClientDataSet file is kept
	fn = fn +"\\MachineAOSCommandDB.XML";																				/// Complete the file name
	XMLDocument1->Active = false;																					/// Will load the commands into the XMLDoc
	if(FileExists(fn))
		XMLUtil->LoadXMLDocument(XMLDocument1,fn);
	else{																																	///Need to create the db
		CreateDB();
		AOSCommandDB->SaveToFile(fn);
		XMLUtil->LoadXMLDocument(XMLDocument1,fn);
	}
	TreeView1->Items->Clear();																						/// Show as a tree view
	XMLUtil->XML2Tree(TreeView1,XMLDocument1);
	ClearBtnClick(this);																									/// Clear the editor panel
	if(!AOSCommandDB->Active){
		AOSCommandDB->Active = true;
	}
	AOSCommandDB->First();
	while(!AOSCommandDB->Eof){																						/// Loop over the commanddb and display contents
		AnsiString s= AOSCommandDB->FieldByName("Command")->AsString;
		editor->AddLine(s);
		AOSCommandDB->Next(); 
	}
}
//---------------------------------------------------------------------------

