#include <vcl.h>
#pragma hdrstop

#include <stdio.h>
#include <string.h>
#include <mem.h>
#include "baXLComm.h"
#include <assert.h>
#include "XMLUtilities.h"
//=====================================================================
// baXLComm Class derived from baSerialPort
//=====================================================================
__fastcall baXLComm::baXLComm(TComponent *Owner):TComponent(Owner)
{
	IsOpen=false;
	LogStatus=false;
	MachineError=false;
	PortHandle = INVALID_HANDLE_VALUE;
	MsgStatus = NULL;
	MachineDB = new TMachineDB(this);								/// The machine object has a database
	XLReadThread = NULL;														/// Make sure no one thinks this is available
	bSimulation = false;														/// Set this to true for software testing or other simulation
}
//---------------------------------------------------------------------
__fastcall baXLComm::~baXLComm()
{
	DeleteThread();
	if(MsgStatus != NULL) delete MsgStatus;
	if(MachineDB)					delete MachineDB;
}
//---------------------------------------------------------------------
bool baXLComm::CloseDevice()
{///Close the Comm port and terminate the read thread
	DeleteThread();
	if(PortHandle!=INVALID_HANDLE_VALUE)
		{///Close the port if it has a valid handle... do not attempt this if the handle is invalid
			Sleep(250);                                                             ///Give the port a chance to settle down from the thread termination
			CloseHandle(PortHandle);                                                ///Close the port
			PortHandle = INVALID_HANDLE_VALUE;                                      ///Make sure the handle is set to invalid
		}
	IsOpen=false;
	return true;
}
//---------------------------------------------------------------------
bool baXLComm::OpenDevice()
{///Open the XL COM port
	AnsiString s;
	if(bSimulation){/// Being run in test mode
		IsOpen 		= true;                                                       ///Set flag true
		return true;                                                            ///Added 8-26-02 to allow operation on computers without appropriate hardware
	}
	Port=Port.LowerCase();
	PortHandle = CreateFile( Port.c_str(),
			GENERIC_READ | GENERIC_WRITE,
			0,
			0,
			OPEN_EXISTING,
			//FILE_ATTRIBUTE_NORMAL,
			FILE_FLAG_OVERLAPPED,                                                   ///asynchronous handling
			0);
	if (PortHandle == INVALID_HANDLE_VALUE )
		{///Uh, oh... couldn't get a valid port handle
			IsOpen=false;
			s=Port+": open Machine comm port failure.";
			MachineDB->AddError(COMM_OPEN_FAIL,s);
			return false;
		}
	if(!SetCommMask(PortHandle,EV_RXCHAR))                                      /// Set the event mask
		{//Handle comm mask error
			CloseHandle(PortHandle);
			PortHandle = INVALID_HANDLE_VALUE;
			IsOpen=false;
			AnsiString s=Port+": set CommMask failure.";
			MachineDB->AddError(COMM_MASK_ERROR,s);
			return false;
		}
	ov.hEvent=CreateEvent (///Create an event object for use in WaitCommEvent
			NULL,                                                                   /// default security attributes
			false,                                                                  /// auto reset event
			false,                                                                  /// not signaled
			NULL);                                                                  /// no name
	assert(ov.hEvent);
	ZeroMemory(&dcb, sizeof(dcb));                                              ///Zero out the data control block memory
	dcb.DCBlength = sizeof(dcb);                                                ///Set the size of the data control block
	if (!BuildCommDCB("19200,N,8,1",&dcb))                                      ///Create the data control block
		{///Unable to build a device control block
			CloseHandle( PortHandle);
			PortHandle = INVALID_HANDLE_VALUE;
			IsOpen=false;
			s=Port+" open failure: unable to build DCB for Machine.";
			MachineDB->AddError(BUILDCOMM,s);
			return false;
		}
	dcb.fRtsControl = RTS_CONTROL_ENABLE;		                                    ///Handshaking- use RTS/DTS
	dcb.fOutxCtsFlow = true;
	dcb.fNull = 0;
	dcb.XoffLim = (BYTESTOREAD - 100);                                          ///Set aside the number of bytes to read
	if (!SetCommState( PortHandle, &dcb))                                       ///Set the Comm using the DCB
		{///Unable to set the comm state with the port handle
			CloseHandle(PortHandle);
			PortHandle =  INVALID_HANDLE_VALUE;
			IsOpen=false;
			s=Port+" open failure: unable to SetCommState with DCB for Machine.";
			MachineDB->AddError(BUILDCOMM,s);
			return false;
		}
	ctmo.ReadIntervalTimeout = MAXDWORD;                                        /// Set Timeouts
	ctmo.ReadTotalTimeoutMultiplier = 0;
	ctmo.ReadTotalTimeoutConstant = 0;
	ctmo.WriteTotalTimeoutMultiplier = 0;
	ctmo.WriteTotalTimeoutConstant = 500;                                       ///Wait 500 ms for a comm timeout when writing to port
	if(!SetCommTimeouts(PortHandle,&ctmo))                                      ///Set the Comm port timeout
		{//Error setting timeout parameters
			CloseHandle(PortHandle);
			PortHandle =  INVALID_HANDLE_VALUE;
			IsOpen=false;
			s=Port+" open failure: unable to SetCommTimeouts for Machine.";
			MachineDB->AddError(COMMTIMER,s);
			return false;
		}
	SetCommMask(PortHandle,EV_RXCHAR);                                          ///Start up ReadThread to monitor the serial port
	CreateThread();																															/// Create a new XLReadThread
	XLReadThread->OurHandle=PortHandle;
	XLReadThread->ov=ov;
	XLReadThread->dwEvtMask=dwEvtMask;
	ResumeThread();
	IsOpen=true;
	return true;
}
//-----------------------------------------------------------------------
int baXLComm::Write(char* buf, size_t len)
{///Write buf to the Comm port. Not threaded, so will try to write the whole thing in one fell swoop
	if(PortHandle == INVALID_HANDLE_VALUE)
		return len;
	LPVOID lpMsgBuf;
  AnsiString s;
	if (buf)
    {///Only write if the buff is valid
       WriteFile(PortHandle,
          buf,
          len,
          &BytesWritten,
          &ov);
    }
	unsigned long BytesXfered;
	GetOverlappedResult(PortHandle,&ov,&BytesXfered,True);
	BytesWritten=BytesXfered;
	return BytesWritten;
}
//---------------------------------------------------------------------
bool baXLComm::SuspendThread()
{///Suspend the read thread
	bool ret=true;
	if(XLReadThread == NULL)
		return ret;																			/// Don't try this on a NULL thread
	if (ret && XLReadThread->Suspended==false)
		XLReadThread->Suspended=true;										/// If it isn't suspended, do so
	return ret;
}
//-----------------------------------------------------------------------
void baXLComm::ResumeThread()
{///Start the read thread
	if(XLReadThread == NULL)
		return;																					/// Don't try this on a NULL thread
	XLReadThread->Resume();
}
//-----------------------------------------------------------------------
void baXLComm::DeleteThread()
{// Safely delete the thread
	if(XLReadThread == NULL)
		return;																					/// Don't try this on a NULL thread
	while(XLReadThread->Suspended)
		XLReadThread->Resume();													/// Dont' try to terminate a suspended thread
	SetCommMask(PortHandle,0);                        /// Disable all comm events before closing read thread
	XLReadThread->Terminate();                      	/// Terminate the thread... free on terminate is set true
	XLReadThread = NULL;                              /// Make sure the pointer is NULL
}
//-----------------------------------------------------------------------
void baXLComm::CreateThread()
{// Safely create the thread
	if(XLReadThread != NULL)
		DeleteThread();																	/// Don't want a duplicate thread!
	XLReadThread = new TXLRead(true);                 /// Create the thread in the suspended state
	XLReadThread->Priority=tpNormal;									/// Works fine with normal priority
}
//-----------------------------------------------------------------------
void __fastcall baXLComm::CreateMsgStatus()
{///Create the message status form
  if(MsgStatus)
    {///There already is one of these, so just make sure it shows
      MsgStatus->BringToFront();
      return;
    }
  MsgStatus = new XLMsgMonFrm(this);
  MsgStatus->Show();
}
//-----------------------------------------------------------------------


