#ifndef baXLCommH
#define baXLCommH
#include <windows.h>
#include "XLReadThread.h"
#include "XLMsgMonFrm.h"
#include "MachineMod.h"

#define RXTIMEOUT           500                                               ///Read timeout 500 ms
#define STX                 2                                                 ///Start message character
#define ETX                 3                                                 ///End message character
#define ACK                 6                                                 ///Acknowledge
#define NAK                 21                                                ///Not acknowledge

#define nullchar            '\0'                                              ///Null character
#define ENDOFMESSAGE        ETX                                               ///Spell out end of message
#define BYTESTOREAD         1000                                              ///Number of bytes to read
#define RXQUEUESIZE         40000U		                                        ///Maximum message size 18 bytes/pt + 24 bytes (for worse case raw data)
															                                                /// 5.000 - 7.000 w/ 0.001 step size is ~ 2000 pts
#define TXQUEUESIZE         1000                                              ///Maximum of 1000 char to be sent
#define MAX_DATASIZE	      RXQUEUESIZE                                       ///Datasize cannot exceed receive queue size
#define CALIBRATION         'C'                                               ///Calibration command/request
#define DATA			          'D'                                               ///Data command/request
#define DAB_ERROR	          'E'                                               ///Data acquisition board error
#define REQUEST		          'R'                                               ///Request data from XL
#define XLStatus			      'X'                                               ///Get the machine status
#define XLSettings 	        'S'                                               ///Get the desired settings
#define FieldService        'A'                                               ///Put the machine in field service mode
#define STARTFUGE           "CL"                                              ///Equivalent of pressing Start button on the XL
#define STOPFUGE            "CX"                                              ///Equivalent of pressing Stop button on the XL
#define SENDSETTINGS        "SS"                                              ///Send over desired settings
#define REQUESTCALIBRATION  "ZZ"                                              ///Get the calibration parms
#define REQUESTSETTINGS     "RS"                                              ///Request the desired settings
#define REQUESTSTATUS       "RX"                                              ///Request the actual values
#define OUTQUEUESIZE	      50                                                ///How many messages may be queued
#define COMMMSG				      57                                                ///???
// COMM ERRORS- Many of these are left over from the old XLA program
#define OPENCOMM		  					50
#define BUILDCOMM		  					51
#define COMMSTATE		  					52
#define COMMTIMER		  					53
#define WRITECOMM		  					54
#define READCOMM		  					55
#define COMMCKSUM		 						56
#define COMMMSG		 							57
#define COMMREQ			 						58
#define COMMTXMSG		 						59
#define COMMCLOSE     					60
#define COMMFAULT		 						61
#define ETX_NOT_FOUND						62
#define SCAN_FILE_ERROR 				63
#define SCANDATA_OVERFLOW 			64
#define MEMORY_ALLOCATION_FAIL 	65
#define FUGE_DIAGNOSTIC					66
#define OUTQUEUE_OVERFLOW				67
#define COMM_OPEN_FAIL					68
#define COMM_MASK_ERROR					69
#define XLERROR		 							100
#include <vcl.h>
#pragma hdrstop

#include "XLData.h"
#include "XLHWData.h"
#include "MachineMod.h"
//=========================================================
class baXLComm :public TComponent
{
private:
	DCB dcb;										/// Device control block of serial port (struct)
	COMMTIMEOUTS ctmo;          /// Timeout values for serial port (struct)
	HANDLE PortHandle;   				/// Handle to serial port
	OVERLAPPED ov;							/// structure for async i/o
	DWORD dwEvtMask;            /// Mask for comm events
 ///Read thread commands
	bool SuspendThread();       /// Suspend the thread safely
	void ResumeThread();				/// Resume the thread
	void DeleteThread();				/// Safely delete the thread
	void CreateThread();				/// Safely create the thread
	unsigned long BytesWritten;
protected:
	int  Write(char* buf, size_t len);
public:
	__fastcall baXLComm(TComponent * Owner);
	__fastcall ~baXLComm();
 //Data Members
	AnsiString Port;     																											/// String used to open port (e.g. "com1")
	bool IsOpen;                                                              /// Set to true if the COMport opened successfully, or if in testing mode
	bool bSimulation;																													/// Set to true if testing software or simulating operations
 /// The COMPort read is threaded
	TXLRead* XLReadThread;
 // Basic Com port methods
	bool CloseDevice();
	bool OpenDevice();
 //Flag to log all status messages
	bool LogStatus;
	bool MachineError;                            ///Set to true if a serious machine error message is set over from the XLA
 ///Message monitor
  XLMsgMonFrm * MsgStatus;                      ///Keep records of messages to/from the machine
  void __fastcall CreateMsgStatus();            ///Create the message status form
	TMachineDB *MachineDB;
};
#endif
